/* cookies.cpp - cookie support for webfetch
 *
 * Copyright (c) 1997-2002 Tony Aiuto
 *
 * To report problems with this tool or to suggest enhancements send mail
 * to tony@aiu.to
 *
 */

#ifndef lint
static char _copyright[] = "@(#) Copyright (c) 1997-2002, Tony Aiuto";
#endif

/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * Edit History:
 *
 * $Log: cookie.cpp $
 * Revision tools_TonyAiuto/6 2004/12/08 17:15:25 TonyAiuto
 * 	5.4.2 release - mostly portability changes for Solaris/macos/sco in
 * 	both the code and tests
 * 
 * Revision tools_TonyAiuto/5 2004/02/23 18:46:14 TonyAiuto
 * 	5.4 release
 * 
 * Revision tools_TonyAiuto/4 2003/08/21 14:33:55 TonyAiuto
 * 	5.2.0 cookie handling changes
 * 
 * Revision tools_TonyAiuto/3 2002/08/07 19:39:52 TonyAiuto
 * 	tweaks for windows
 * 
 * Revision tools_TonyAiuto/2 2002/08/07 17:40:05 TonyAiuto
 * 	add comment and version
 * 
 */

#ifndef lint
static char rcsid[] = "@(#) $Id: cookie.cpp tools_TonyAiuto/6 2004/12/08 17:15:25 TonyAiuto $";
#endif


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include <stdlib.h>

#if defined(WIN32)

#include <windows.h>
#include <shlwapi.h>
#define strncasecmp(a, b, n) ::StrCmpNI(a, b, n)
#define strcasecmp(a, b) ::StrCmpI(a, b)

#endif

#include "osdep.h"

#include "cookie.h"

Cookie::Cookie()
{
	name = value = domain = path = expires = NULL;
}

Cookie::~Cookie()
{
	if(comment) free(comment);
	if(name) free(name);
	if(value) free(value);
	if(domain) free(domain);
	if(path) free(path);
	if(expires) free(expires);
	if(secure) free(secure);
}

Cookie::Cookie(const char *rawtext)
{
	const char	*s = rawtext;
	const char	*mark;
	int	toklen;

	comment = name = value = domain = path = expires = secure = NULL;
	version = 1;

	/* sample
	 * skin=; domain=.amazon.com; path=/; 
	 *		expires=Wed, 01-Aug-01 12:00:00 GMT
Set-Cookie: visit=20020807; expires=Sat, 17-Aug-2002 14:25:49 GMT; path=/; inc=0
	 */

	/* we ignore everything but the actual value for now.
	 * That's a reasonable choice because we assume if we are getting
	 * cookies, then it's only becasue they should be sent back
	 */


	while(isspace(*s)) s++;
	mark = s;
	while(*s && *s != '=') s++;
	toklen = s - mark;
	if(*s != '=' || toklen <= 0) {
		fprintf(stderr, "Bad Cookie from server: %s\n", rawtext);
		return;
	}
	s++;
	name = (char *) malloc(toklen+1);
	memcpy(name, mark, toklen);
	name[toklen] = '\0';

	mark=s;
	while(*s && *s != ';') s++;
	toklen = s - mark;
	value = (char *) malloc(toklen+1);
	memcpy(value, mark, toklen);
	value[toklen] = '\0';

	if(*s) s++;		// skip terminator
	while(*s) {
		while(isspace(*s)) s++;

		if(strncasecmp(s, "domain=", 7) == 0) {
			s += 7;
			mark = s;
			while(*s && *s != ';' && *s >= ' ') s++;
			toklen = s - mark;
			domain = (char *) malloc(toklen+1);
			memcpy(domain, mark, toklen);
			domain[toklen] = '\0';
			if(*s == ';') s++;
		} else if(strncasecmp(s, "path=", 5) == 0) {
			s += 5;
			mark = s;
			while(*s && *s != ';' && *s >= ' ') s++;
			toklen = s - mark;
			path = (char *) malloc(toklen+1);
			memcpy(path, mark, toklen);
			path[toklen] = '\0';
			if(*s == ';') s++;
		} else if(strncasecmp(s, "expires=", 8) == 0) {
			s += 8;
			mark = s;
			while(*s && *s != ';' && *s >= ' ') s++;
			toklen = s - mark;
			expires = (char *) malloc(toklen+1);
			memcpy(expires, mark, toklen);
			expires[toklen] = '\0';
			if(*s == ';') s++;
		} else if(strncasecmp(s, "comment=", 8) == 0) {
			s += 8;
			mark = s;
			while(*s && *s != ';' && *s >= ' ') s++;
			toklen = s - mark;
			comment = (char *) malloc(toklen+1);
			memcpy(expires, mark, toklen);
			comment[toklen] = '\0';
			if(*s == ';') s++;
		} else if(strncasecmp(s, "version=", 8) == 0) {
			s += 8;
			version = 0;
			while(*s && *s != ';' && *s >= ' ') {
				if(isdigit(*s)) {
					version = version * 10 + *s - '0';
				}
			}
			if(*s == ';') s++;
		} else {
			// we got something we don't expect
			while(*s && *s != ';' && *s >= ' ') s++;
		}
	}
	if(path == NULL || *path == '\0') {
		path = strdup("/");
	}
}



CookieJar::CookieJar()
{
	allocCookies = _COOKIE_JAR_CAPACITY;
	ncookies = 0;
	cookies = new Cookie * [allocCookies];
	cookieJarFileName = NULL;
}

CookieJar::~CookieJar()
{
	for(int i = 0; i < ncookies; i++) delete cookies[i];
	delete [] cookies;
	::free( cookieJarFileName );
}


/* Deal with cookies from server.
 *
 * We keep an in core list of them and send them on any subsequent calls
 * We should also write them to cookie jar
 */

void CookieJar::handleCookie(const char *rawtext, const char *defdomain)
{
	// fprintf(stderr, "Set-Cookie: %s", rawtext);
	Cookie *c = new Cookie(rawtext);
	if(c->name == NULL) {
		delete c;
		return;
	}

	// Check for a delete cookie request
	if(c->value == NULL) {
		if(deleteCookie(c)) {
			if(cookieJarFileName) this->save(cookieJarFileName);
		}
		delete c;
		return;
	}

	// fill in cookie defaults
	if(c->domain == NULL || *c->domain == '\0') {
		c->domain = strdup(defdomain);
	}

	if(addCookie(c)) {
		if(cookieJarFileName) this->save(cookieJarFileName);
	}
}

void CookieJar::load(const char * _cookieJarFileName)
{
	char	line[500];

	cookieJarFileName = strdup(_cookieJarFileName);
	FILE *jar = fopen(cookieJarFileName, "r");
	if(jar == NULL) return;

	while(fgets(line, sizeof(line), jar)) {
		int l = strlen(line) - 1;
		while(l >= 0 && isspace(line[l])) line[l--] = '\0';
		if(line[0] != '\0') {
			Cookie *c = new Cookie(line);
			if(c->domain == NULL || *c->domain == '\0') {
				c->domain = strdup("*");
			}
			cookies[ncookies++] = c;
		}
	}
//	if(verbose > 0) {
//		fprintf(stderr, "loaded %d cookies\n", ncookies);
//	}
	fclose(jar);
}


void CookieJar::save(const char * fname)
{
	FILE *jar = fopen(fname, "w");
	if(jar == NULL) return;

	for(int i = 0; i < ncookies; i++) {
		Cookie *x = cookies[i];

		fprintf(jar, "%s=%s;", x->name, x->value);
		if(x->domain != NULL) {
			fprintf(jar, " domain=%s;", x->domain);
		}
		if(x->path != NULL) {
			fprintf(jar, " path=%s;", x->path);
		} else {
			fprintf(jar, " path=/;", x->path);
		}
		if(x->expires != NULL) {
			fprintf(jar, " expires=%s", x->expires);
		}
		fprintf(jar, "\n");
	}
	fclose(jar);
}

int CookieJar::addCookie(Cookie *c)
{
	// Replace it if we can
	for(int i = 0; i < ncookies; i++) {
		if(strcmp(cookies[i]->name, c->name) == 0) {
			delete cookies[i];
			cookies[i] = c;
			return 1;
		}
	}

	// have to add it
	if(ncookies >= allocCookies) {
		fprintf(stderr,
		"Out of cookie space: fix and send patch back to tony@aiu.to. :-)\n");
		return 0;
	}

	cookies[ncookies++] = c;
	return 1;
}



int CookieJar::deleteCookie(Cookie *c)
{
	// printf("Deleting cookie %s\n", c->name);
	for(int i = 0; i < ncookies; i++) {
		if(strcmp(cookies[i]->name, c->name) == 0) {
			delete cookies[i];	// fling the acursed
			ncookies -= 1;		// shuffle down
			for(; i < ncookies; i++) {
				cookies[i] = cookies[i+1];
			}
			cookies[ncookies] = NULL;  // leave no garbage pointers
			return 1;
		}
	}
	return 0;
}
