#!/bin/sh

# copysite - copy a network of files from a website
# 
# usage:  copysite www.aiu.to index.html
# NOTE: do NOT include the leading slash on the base html file
#
# Side effects:
#	It uses a file called DONE to mark which pages it got before
#	You may want to clean this out periodically


# keep name of program for recursive call
COPYSITE=$0

# set options to defaults
OPTIONS=""
FETCHOPTS="-R"
DBG=":"
verbose=0
TODO=""
AUTHKEY=""
# set to 1 if you do not want to preserve html files
noSaveHTML=0

while : ; do
	case $1 in
	-d )
		shift
		DBG=""
		OPTIONS="$OPTIONS -d"
		;;
	-a )
		shift
		AUTHKEY=$1
		shift
		OPTIONS="$OPTIONS -a $AUTHKEY"
		FETCHOPTS="$FETCHOPTS -a $AUTHKEY"
		;;
	-v )
		shift
		verbose=1
		OPTIONS="$OPTIONS -v"
		;;
	-q )
		shift
		verbose=0
		OPTIONS="$OPTIONS -q"
		;;

	-TODO  )
		shift
		TODO=$1
		shift
		;;
	*)
		break;
	esac
done


$DBG echo Debugging enabled

if [ $# -ne 2 ] ; then
	echo usage: $COPYSITE '[-vq] site basefile'
	exit 1
fi

SITE=$1
BASE=$2
# Strip leading / from file name
BASE=`echo $BASE | sed -e 's#^/##'`


# Step one - retrieve target file
out=$BASE
case $BASE in
*/ )
	out="${BASE}index.html"
	;;
esac

if [ -f $BASE ] ; then
	# use existing
	if [ $verbose -ne 0 ] ; then
		echo Using existing $BASE
	fi
else
	## mkdir -p `dirname $BASE`
	## GET http://$SITE/$BASE >$BASE
	CMD="webfetch $FETCHOPTS -o $out -d -r http://$SITE/$BASE http://$SITE/$BASE"
	if [ $verbose -ne 0 ] ; then
		echo $CMD
	fi
	$CMD
fi

$DBG # check it out
$DBG xterm -e /bin/vi $out

# A file is DONE when we do the getlinks on it
echo $SITE $out >>DONE
sort -u -o DONE DONE

# extract links
#	eliminate things we do not follow and fix up some funny things
#OBS baseref=`dirname $out`
#OBS getlinks -s $SITE -b $baseref $out . sed -e '
getlinks -s $SITE $out | sed -e '
/mailto:/d
/ftp:/d
/cgi-bin/d
/mailform.php/d
s#  *# #
s# \./# #
s# /# #
s/#.*$//
' | sort -u | comm -13 DONE - >/tmp/$$.list 

if [ $noSaveHTML -ne 0 ] ; then
	rm $out
fi

if [ ! -s /tmp/$$.list ] ; then
	if [ $verbose -ne 0 ] ; then
		echo "No new files in $BASE"
	fi
	rm -f /tmp/$$.*
	exit 0
fi

trap '/bin/rm -f /tmp/$$.* ; exit 0' 0 1 2 3 15

# Does not work yet
# Set up overall todo list
#if [ -z "$TODO" ] ; then
#	TODO=/tmp/$$.todo
#fi
#OPTIONS="$OPTIONS -TODO $TODO"
#touch $TODO

# let user edit the new list of things to get
xterm -e /bin/vi /tmp/$$.list


# Pass 1 - fetch the files in this list

cat /tmp/$$.list | while read site file  ; do

	case $site in 
	CMT )
		echo $file
		continue
		;;
	esac

	# for files ending in / we guess to output it to index.html
	out=$file
	case $file in
	*/ )
		out="${file}index.html"
		;;
	esac

	# We check the DONE list because we can use it to mark files that
	# we don't want to retreive as being done.
	if (grep -q "$site $out\$" DONE) ; then
		echo DONE $file before
		continue;
	fi

	case $file in
	*.htm | *.html | *.HTM | *.HTML | */ )
			# mark for later rescan
		echo $site $out >>/tmp/$$.redo
		# if we are not saving .html files, then we only have to mark
		# it for rescan
		if [ $noSaveHTML -ne 0 ] ; then
			continue
		fi
		;;
	esac

	# do not retreive files that we have
	if [ -f $out ] ; then
		echo Did $file before
		continue;
	fi

	# snag the file

	## mkdir -p `dirname $file` >/dev/null
	## CMD="GET http://$site/$file >$out"
	CMD="webfetch $FETCHOPTS -d -o $out -r http://$site/$BASE http://$site/$file"
	if [ $verbose -ne 0 ] ; then
		echo $CMD
	fi
	$CMD
done


# pass 2 - recurse down the HTML files

if [ -f /tmp/$$.redo ] ; then

	# do not do files that are marked as DONE
	sort -u -o /tmp/$$.redo /tmp/$$.redo
	if [ $verbose -ne 0 ] ; then
		echo Eliminating these files because we did them:
		comm -12 DONE /tmp/$$.redo
	fi
	comm -13 DONE /tmp/$$.redo >/tmp/$$.list
	rm /tmp/$$.redo

	#DBG xterm -e /bin/vi /tmp/$$.list

##	# now elimintate items that are already marked as todo
##	if [ $verbose -ne 0 ] ; then
##		echo Eliminating files because we are on TODO list - $TODO
##		comm -12 $TODO /tmp/$$.list
##	fi
##	comm -13 $TODO /tmp/$$.list >/tmp/$$.redo
##
##	# and add to the list of things we are going to do
##	cat /tmp/$$.redo >>$TODO
##	sort -u -o $TODO $TODO

##	if [ $verbose -gt 1 ] ; then
##		echo We are left with a list of:
##		cat /tmp/$$.redo
##	fi


	cat /tmp/$$.list | while read site file  ; do
		if (grep -q "$site $file\$" DONE) ; then
			echo Did $file before - should not see this comment
			continue;
		fi

		if [ $verbose -ne 0 ] ; then
			echo $COPYSITE $OPTIONS $site $file
		fi
		$COPYSITE $OPTIONS $site $file </dev/tty >/dev/tty
	done
fi

rm -f /tmp/$$.*

exit 0
