/* Unix emulation functions needed for windows programs
 *
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <windows.h>

/*
 * getopt - get option letter from argv
 *	This software is in the public domain
 *	Originally written by Henry Spenser at the U. of Toronto
 */

extern "C" {
char	*optarg;	/* Global argument pointer. */
int	optind = 0;	/* Global argv index. */


int getopt(int argc, char * const argv[], const char *optstring)
{
	register char c;
	register char *place;
	static char *scan = NULL;	/* Private scan pointer. */

	optarg = NULL;

	if (scan == NULL || *scan == '\0') {
		if (optind == 0)
			optind++;
	
		if (optind >= argc || argv[optind][0] != '-' || argv[optind][1] == '\0')
			return EOF;
		if (strcmp(argv[optind], "--")==0) {
			optind++;
			return EOF;
		}
	
		scan = argv[optind]+1;
		optind++;
	}

	c = *scan++;
	place = strchr(optstring, c);

	if (place == NULL || c == ':') {
		fprintf(stderr, "%s: unknown option -%c\n", argv[0], c);
		return '?';
	}

	place++;
	if (*place == ':') {
		if (*scan != '\0') {
			optarg = scan;
			scan = NULL;
		} else {
			optarg = argv[optind];
			optind++;
		}
	}

	return c;
}


int strncasecmp(const char *a, const char *b, int len)
{
	int	a1, b1, x;
	while(len-- > 0) {
		a1 = *a++;
		if(isupper(a1)) a1 = tolower(a1);
		b1 = *b++;
		if(isupper(b1)) b1 = tolower(b1);
		x = a1 - b1;
		if(x != 0) return x;
	}
	return 0;
}


int gettimeofday(struct timeval *tv, struct timezone *dummy)
{
#if NOT_GOOD
	// This is a bad approximation.  It is only accurate to the second
	time_t	t; 
	time(&t);
	tv->tv_sec = t;
	tv->tv_usec = 0;
#endif
	SYSTEMTIME	st;
	GetSystemTime(&st);
	tv->tv_sec = st.wSecond + 60*(st.wMinute + 60*(st.wHour + 24*st.wDay));
	tv->tv_usec = st.wMilliseconds * 1000;
	return 0;
}

} /* end extern "C" */

