/* webfetch.cpp - fetch a file from an http server
 *
 * Copyright (c) 1997-2002 Tony Aiuto
 *
 * To report problems with this tool or to suggest enhancements send mail
 * to tony@aiu.to
 *
 */

#ifndef lint
static char _copyright[] = "@(#) Copyright (c) 1997-2002, Tony Aiuto";
#endif

/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * Edit History:
 *
 * $Log: main.cpp $
 * Revision tools_TonyAiuto/15 2006/02/01 15:29:05 TonyAiuto
 * 	get version stamp in user-agent from Makfile
 * 
 * Revision tools_TonyAiuto/14 2004/12/08 17:15:25 TonyAiuto
 * 	5.4.2 release - mostly portability changes for Solaris/macos/sco in
 * 	both the code and tests
 * 
 * Revision tools_TonyAiuto/13 2004/02/23 18:46:14 TonyAiuto
 * 	5.4 release
 * 
 * Revision tools_TonyAiuto/12 2004/01/07 17:51:30 TonyAiuto
 * 	5.3.4 savepoint after HPUX and windows initial tweaks
 * 
 * Revision tools_TonyAiuto/11 2003/11/21 03:46:14 TonyAiuto
 * 	changes for 5.3.0
 * 
 * Revision tools_TonyAiuto/10 2003/11/13 20:30:19 TonyAiuto
 * 	interim checkpoint
 * 
 * Revision tools_TonyAiuto/9 2003/11/13 17:07:02 TonyAiuto
 * 	Eliminated need for globals commandType and proxy_auth
 * 
 * Revision tools_TonyAiuto/8 2003/10/26 03:15:18 TonyAiuto
 * 	5.2.1 - move placement of tobase64
 * 
 * Revision tools_TonyAiuto/6 2003/08/21 13:33:39 TonyAiuto
 * 	fix bug where "-o file -C head" would not work, output was always to stdout.
 * 
 * Revision tools_TonyAiuto/5 2003/01/17 17:11:06 TonyAiuto
 * 	automate version stamping to windows
 * 
 * Revision tools_TonyAiuto/4 2002/10/17 19:16:42 TonyAiuto
 * 	fix useragent bug and better tests
 * 
 * Revision tools_TonyAiuto/3 2002/10/17 18:08:22 TonyAiuto
 * 	5.1.0 release
 * 
 * Revision tools_TonyAiuto/2 2002/08/07 19:58:25 TonyAiuto
 * 	5.0.0 beta
 * 
 * Revision tools_TonyAiuto/19 2001/11/15 17:42:03 TonyAiuto
 * 	4.0.1
 * 
 * Revision tools_TonyAiuto/18 2001/10/26 21:39:26 TonyAiuto
 * 	4.0.1 - cookie jars + internal improvements
 * 
 * Revision tools_TonyAiuto/17 2001/05/01 19:19:37 TonyAiuto
 * 	dusting and cleaning
 * 
 * Revision tools_TonyAiuto/16 2001/01/18 06:44:34 TonyAiuto
 * 	windows fixes
 * 
 * Revision tools_TonyAiuto/15 2000/11/15 20:45:16 TonyAiuto
 * 	add rpm packaging
 * 
 * Revision tools_TonyAiuto/14 2000/10/13 15:34:02 TonyAiuto
 * 	add -C op  option to get HEAD
 * 
 * Revision tools_TonyAiuto/13 2000/10/04 21:15:48 TonyAiuto
 * 	WARNING!!!!!  Meaning of -s reversed.   This will prevent a lot
 * 	of accidental damage when you use webfetch from the command line
 * 	but will break most existing scripts.
 * 	
 * 	add timeout so it does not hang forever.
 * 
 * Revision tools_TonyAiuto/12 2000/05/03 20:33:14 TonyAiuto
 * 	add -R option to follow redirects
 * 
 * Revision tools_TonyAiuto/11 2000/04/14 19:23:30 TonyAiuto
 * 	Version 2.2.0
 * 	Add -D option to do delete rather than get
 * 	Add -f user option to add 'From' header
 * 
 * Revision tools_TonyAiuto/10 2000/03/27 19:06:29 TonyAiuto
 * 	Support for SOCKS4 and windows
 * 
 * Revision tools_TonyAiuto/8 2000/03/24 17:47:38 TonyAiuto
 * 	re-keep current source
 * 
 * 
 * --- note versions 8 and 9 are repeated because the backing store got
 * --- restored to an earlier copy   The newer source was not lost, so
 * --- it just got re-checked in - tony - Fri Mar 24 12:46:59 EST 2000
 *
 * Mon Mar 20 10:48:20 EST 2000
 * Add little fixes for MS - HTTP proxy
 *
 * Revision tools_TonyAiuto/9 1999/10/11 17:51:16 TonyAiuto
 * 	bug fix in chunked reading, print cookies
 * 
 * Revision tools_TonyAiuto/8 1999/09/09 20:46:34 TonyAiuto
 * 	new command syntax  webfetch [options] URL ...  (old syntax still works too)
 * 	change -p option to take args for POST command
 * 	handle chunked transfer encoding (usually from CGI pages)
 * 	improve IO handling
 * 
 * Revision tools_TonyAiuto/7 1999/07/26 16:46:09 TonyAiuto
 * 	fix stupid core dump
 * 
 * Revision tools_TonyAiuto/6 1999/05/27 03:15:14 TonyAiuto
 * 	Add -a option for authentication. Make -d option work in conjunction with -o
 * 
 * Revision tools_TonyAiuto/5 1999/04/29 01:54:28 TonyAiuto
 * 	http proxy
 * 
 * Revision tools_TonyAiuto/4 1999/01/16 22:49:31 TonyAiuto
 * 	add "-H host" option
 * 
 * Revision tools_TonyAiuto/3 1999/01/13 18:06:51 TonyAiuto
 * 	add -h option for headers
 * 
 * Revision tools_TonyAiuto/2 1999/01/13 15:58:04 TonyAiuto
 * 	minor fixes
 * 
 * Revision 1.14  1999/01/13 15:56:19  tony
 * allow host:port syntax on host name
 * fix up read loop so it does not wait for timeout on very short responses
 *
 * Revision 1.13  1998/09/16 16:46:03  tony
 * remove debug message
 *
 * Revision 1.12  1998/09/16 16:43:11  tony
 * Fix up file names with %xx in them
 * and remove ~, !, space and quote characters
 *
 * Revision 1.11  1998/07/22 20:49:19  tony
 * use Content-Length header to know when file ends
 *
 * Revision 1.10  1998/06/18 02:26:05  tony
 * fix legal crap
 *
 * Revision 1.9  1998/05/29 17:53:20  tony
 * remove annoying debug message
 *
 * Revision 1.8  1998/05/07 17:55:22  tony
 * Add -d option to make directories
 *
 * Revision 1.7  1998/05/07 17:39:53  tony
 * Convert to use HTTP 1.1
 *
 * Revision 1.6  1998/03/02 16:48:49  tony
 * better eof handling
 *
 * Revision 1.5  1998/02/25 17:25:22  tony
 * missing a break statement
 *
 * Revision 1.4  1998/02/25 17:06:01  tony
 * add -c cookie option
 *
 * Revision 1.3  1998/02/18 17:14:05  tony
 * make out file close on signalled shutdown
 * fix to logic determining when a header ends
 *
 * Revision 1.2  1997/12/30 19:43:45  tony
 * Add -r option
 *
 * Revision 1.1  1997/12/24 17:07:11  tony
 * Initial revision
 *
 *
 * 12/22/97 tony - created
 */

#ifndef lint
static char rcsid[] = "@(#) $Id: main.cpp tools_TonyAiuto/15 2006/02/01 15:29:05 TonyAiuto $";
#endif


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include <stdlib.h>

// Bring in the platform specific indcludes

#if defined(WIN32)

#include <winsock2.h>
#include <shlwapi.h>
#define strncasecmp(a, b, n) ::StrCmpNI(a, b, n)
#define strcasecmp(a, b) ::StrCmpI(a, b)

#else

#include <unistd.h>
#include <sys/utsname.h>

#endif

#include <signal.h>

#if defined(WIN32) || defined(sun)
extern "C" {
extern int getopt(...);
extern int	optind;
extern char	*optarg;
}
#elif defined(linux)
#include <getopt.h>
#endif

extern	int	optind;
extern	char	*optarg;


#include <osdep.h>
#include <cookie.h>
#include <proxy.h>
#include <webfetch.h>
#include <webconstants.h>

#if HAVE_SSL > 0
#include <ssloptions.h>
#endif


/* we have a list of globals here which mostly represent program
 * options.  Ideally they all should be in the "Main" class, but
 * I haven't gotten to that yet.
 */

int		verbose = 0;


static	const char	*language;	// used by user agent stuff
static	Webfetch	*wf = NULL;	// the fetcher
static	int		showHeaders = 0;	// should we write headers


static	UploadData	*postData = NULL;	// args for HTTP POST
static	char	*socksProxy = NULL;	// SOCKS Proxy information
static	int	httpCommand = CMD_GET;
static	int	printStatus = 0;	// print error & HTTP status codes


// pick the way to handle the data
static	PrintingDataConsumer	printData;
static	FileDataConsumer	saveData;

#if HAVE_SSL > 0
static	SSLOptions *ssloptions;
#endif

static void usage()
{
	printf("webfetch %s\n", VERSION);
	printf("usage: webfetch [options] URL ...\n");
	printf("usage: webfetch [options] @commandFile ...\n");
	printf("usage: webfetch [options] site resource ... (deprecated)\n");
	printf("options:\n");
	printf("\t-A user:pass\tsend Proxy-Authentication header.\n");
	printf("\t-a user:pass\tsend Authentication header.\n");
	printf("\t-C {get,head,delete,post}\tset http Command.\n");
	printf("\t-c cookie\tsend a Cookie.\n");
	printf("\t-d\t\tcreate Directories as needed. (implies -s)\n");
	printf("\t-e\t\tprint error and & HTTP status code on stderr\n");
	printf("\t\t\tSaved resonses are written to the full URI path.\n");
	printf("\t-f user\t\tset the From header Default: none\n");
	printf("\t-h\t\tshow HTTP response Headers.\n");
	printf("\t-j file\t\tuse file as a cookie Jar.\n");
	printf("\t-l language\tset Accept-Language header.\n");
	printf("\t-m host\t\tSend request to this host, using virtual host from the URL.\n");
	printf("\t-n contentType\tSpecify Content-Type header to send with post data.\n\t\t\tDefault: application/x-www-form-urlencoded\n");
	printf("\t-o outfile\twrite page to specified file.  See -d & -o (implies -s)\n");
	printf("\t-P host{:port}\thse HTTP Proxy through specified host and port.\n");
	printf("\t-p args\t\tPOST args to url\n");
	printf("\t-R\t\tfollow Redirect responses from server.\n");
	printf("\t-r referingpage\tset Referer field of HTTP header.\n");
	printf("\t-S host{:port}\tuse Socks proxy through specified host and port.\n");
	printf("\t-s\t\tSave response in a file whose name matches the last\n");
	printf("\t\t\telement of the URI.  (otherwise write to standard out)\n");
	printf("\t-t n\t\tread Timeout in seconds (default 60).\n");
	printf("\t-U userAgent\tSpecify your own User agent exactly.\n");
	printf("\t-u {aol{N},ie{N},netscape,linux,nt,95,98}\tspecify what User agent\n");
	printf("\t\t\tto pretend to be.  Specifiers may be combined.\n");
	// printf("\t\t\tlinux, nt, 95, 98, netscape, ie{N}, aol{N}\n");
	printf("\t-D\t\t(deprecated) send a DELETE request rather than a GET.\n");
#if HAVE_SSL > 0
	printf("ssl options:\n");
	printf("\t-k allow-expired\tallow self signed keys\n");
	printf("\t-k allow-self-signed\tallow self signed keys\n");
	printf("\t-k disallow-self-signed\tdisallow self signed keys (default)\n");
	printf("\t-k allow-untrusted\tallow untrusted CAs\n");
	printf("\t-k disallow-untrusted\tdisallow untrusted CAs\n");
	printf("\t-k certfile=file\tspecify trusted certificate file\n");
	printf("\t-k certpath=file\tspecify trusted certificate directory path\n");
	printf("\t-k certkey=keyfile\tspecify client key file\n");
#endif

	printf("\nMail bug reports and suggestions to <source@aiu.to>.\n");
	exit(0);
}


void shutdown(int sig)
{
#if defined(unix)
	if(sig == SIGALRM) fprintf(stderr, "Timeout\n");
#endif
	// printf("Requested shutdown\n");
	if(wf != NULL) wf->shutdown();
	exit(0);
}



/* build the userAgent string based on some specifiers
 */
static char *decodeUserAgent(const char *ua)
{
	char	ret[200];
	char	collectVersion[20];
	int	ncollectVersion = 0;
	const char	*version = "4.5";	// Mozilla version (usually a lie)
#define	OS_LINUX	1
#define	OS_WINNT	2
#define	OS_WIN95	3
#define	OS_WIN98	4
	int	ostype = 0;

	int	isIE = 0;
	int	isAOL = 0;
	char	minorVersion[20];
	int	nminorVersion = 0;
	minorVersion[0] = '\0';

	while(*ua) {
		if(*ua == ',') {
			ua += 1;
		} else if(*ua == '.') {
			ua += 1;
			nminorVersion = 0;
			while(isdigit(*ua)) {
				minorVersion[nminorVersion++] = *ua;
				ua++;
			}
		} else if(strncasecmp(ua, "ie", 2)==0) {
			ua += 2;
			isIE = 4;
			version = "4.0";
			strcpy(minorVersion, "01");
			nminorVersion = 2;
			if(isdigit(*ua)) {
				isIE=0;
				while(isdigit(*ua)) {
					isIE = isIE * 10 + (*ua) - '0';
					ua++;
				}
			}
		} else if(strncasecmp(ua, "netscape", 8)==0) {
			ua += 8;
			version = "4.5";
			isIE = 0;
			isAOL = 0;
		} else if(strncasecmp(ua, "aol", 3)==0) {
			ua += 3;
			isAOL = 7;
			if(isdigit(*ua)) {
				isAOL=0;
				while(isdigit(*ua)) {
					isAOL = isAOL * 10 + (*ua) - '0';
					ua++;
				}
			}
			if(isIE == 0) {
				isIE = 6;
				version = "4.0";
			}
		} else if(strncasecmp(ua, "linux", 5) == 0) {
			ua += 5;
			ostype = OS_LINUX;
		} else if(strncasecmp(ua, "nt", 2) == 0) {
			ua += 2;
			ostype = OS_WINNT;
		} else if(strncasecmp(ua, "95", 2) == 0) {
			ua += 2;
			ostype = OS_WIN95;
		} else if(strncasecmp(ua, "98", 2) == 0) {
			ua += 2;
			ostype = OS_WIN98;
		} else if(strncasecmp(ua, "xp", 2) == 0) {
			ua += 2;
			ostype = OS_WINNT;
		} else if(isdigit(*ua) || *ua == '.') {
			collectVersion[ncollectVersion++] = *ua;
			ua += 1;
		}
	}
	minorVersion[nminorVersion] = '\0';
	if(ostype == 0) ostype = OS_WIN98;

	strcpy(ret, "Mozilla/");
	if(ncollectVersion > 0) {
		collectVersion[ncollectVersion] = '\0';
		strcat(ret, collectVersion);
	} else {
		strcat(ret, version);
	}

#if ACTUAL_SAMPLES
IE 4 on NT: Mozilla/4.0 (compatible; MSIE 4.01; Windows NT);
IE 5 on 98: Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt);
IE 5 on NG: Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt);
Netscape 4.5 on linux:	Mozilla/4.6 [en] (X11; U; Linux 2.0.34 i486)
Mozilla/4.0 (compatible; MSIE 5.0; AOL 4.0; Windows 98; DigExt)|1
Mozilla/4.0 (compatible; MSIE 5.01; AOL 5.0; Windows 98; Wowzer.com)|1
Mozilla/4.0 (compatible; MSIE 5.01; AOL 6.0; Windows 98; BCD2000)|1
Mozilla/4.0 (compatible; MSIE 5.01; AOL 7.0; Windows NT 5.0; DigExt)|2
Mozilla/4.0 (compatible; MSIE 5.0; AOL 6.0; Windows 95; DigExt)|1

#endif

	if(isIE > 10) {
		sprintf(minorVersion, "%d", isIE % 10);
		isIE /= 10;
	}
	if(isIE > 0) {
		strcat(ret, " (compatible");
		if(isIE == 5) {
			sprintf(&ret[strlen(ret)], "; MSIE %d.%s", isIE,
					minorVersion);
		}
		if(isAOL > 0) {
			sprintf(&ret[strlen(ret)], "; AOL %d.%s", isAOL,
					minorVersion);
		}
		switch(ostype) {
		default:
		case OS_WINNT:
			strcat(ret, "; Windows NT"); break;
		case OS_WIN95:
			strcat(ret, "; Windows 95"); break;
		case OS_WIN98:
			strcat(ret, "; Windows 98"); break;
		case OS_LINUX:
			strcat(ret, "; linux"); break;
		}
		if(isIE == 5) strcat(ret, "; DigExt");
		strcat(ret, ")");
	} else {
		// Netscape style includes the language

		if(language) {
			strcat(ret, " [");
			strcat(ret, language);
			strcat(ret, "]");
		}

		switch(ostype) {
		default:
#if defined(unix)
			struct utsname	hostinfo;
			if(uname(&hostinfo) == 0) {
				strcat(ret, " (");
				strcat(ret, hostinfo.sysname);
				strcat(ret, " ");
				strcat(ret, hostinfo.release);
				strcat(ret, " ");
				strcat(ret, hostinfo.machine);
				strcat(ret, ")");
			}
#elif defined(WIN32)
			// Only windoze could be this brain dead in version
			// identification
			strcat(ret, "windoze");

			OSVERSIONINFO verInfo;
			verInfo.dwOSVersionInfoSize = sizeof(verInfo);
			if(GetVersionEx(&verInfo) == TRUE) {
				if(verInfo.dwPlatformId == 
						VER_PLATFORM_WIN32_NT) {
					strcat(ret, " NT");
				} else if(verInfo.dwPlatformId == 
						VER_PLATFORM_WIN32_WINDOWS) {
					strcat(ret, " ");
					if(verInfo.dwMinorVersion == 0) {
						strcat(ret, " 95");
					} else {
						strcat(ret, " 98");
					}
				} else if(verInfo.dwPlatformId == 
						VER_PLATFORM_WIN32s) {
					strcat(ret, " 3.1");
				}
				sprintf(collectVersion, "%d.%d",
					verInfo.dwMajorVersion,
					verInfo.dwMinorVersion);
				strcat(ret,  collectVersion);
			}
#endif
			break;

		case OS_LINUX:
			strcat(ret, " (X11; U; Linux 2.0.34 i486"); break;
		case OS_WINNT:
			strcat(ret, " (WinNT; U)"); break;
		case OS_WIN95:
			strcat(ret, " (Win95; U)"); break;
		case OS_WIN98:
			strcat(ret, " (Win98; U)"); break;
		}
	}


	return strdup(ret);
}


static void dealWithHeader(Webfetch * /* fetcher */, const char *headerbuffer, int n)
{
	if(verbose > 1) {
		printf("Got header (len %d) --->'%s'<---\n", n, headerbuffer);
	}

	if(showHeaders) {
		fwrite(headerbuffer, 1, n, stdout);
	}
}



static int processArg(Webfetch *wf, CookieJar *cookies, int what, const char *arg)
{
	// printf("processArg(%c, '%s'\n", what, arg ? arg : "<none>");
	switch(what) {
	case 'a':
		wf->setAuthorization(arg);
		break;
	case 'A':
		wf->setProxyAuthorization(arg);
		break;
	case 'C':
		if(strcasecmp(arg, "head") == 0) {
			httpCommand = CMD_HEAD;
		} else if(strcasecmp(arg, "delete") == 0) {
			httpCommand = CMD_DELETE;
		} else if(strcasecmp(arg, "post") == 0) {
			httpCommand = CMD_POST;
		} else if(strcasecmp(arg, "put") == 0) {
			httpCommand = CMD_PUT;
		} else if(strcasecmp(arg, "propfind") == 0) {
			httpCommand = CMD_PROPFIND;
		} else if(strcasecmp(arg, "propdel") == 0) {
			httpCommand = CMD_PROPDEL;
		} else if(strcasecmp(arg, "report") == 0) {
			httpCommand = CMD_REPORT;
		}
		break;
	case 'c':
		wf->setCookie(arg);
		break;
	case 'D':
		httpCommand = CMD_DELETE;
		break;
	case 'd':
		saveData.setCreateDirectories(1);
		wf->setDataConsumer(&saveData);
		break;
	case 'e':
		printStatus = 1;
		break;
	case 'f':
		wf->setFrom(arg);
		break;
	case 'H':
		fprintf(stderr, "-H host has been removed.  Instead of '-H host http://machine/' use '-m machine http://host/'\n");
		exit(1);

	case 'h':
		showHeaders = 1;
		break;
	case 'j':
		cookies->load(arg);
		break;
	case 'l':
		language = strdup(arg);
		wf->setLanguage(arg);
		break;
	case 'm':
		wf->setConnectHost(arg);
		break;
#if MOZILLA_OPTION
	case 'M':
		if(arg != NULL && *arg != '\0') {
			userAgent = new char[12 + strlen(arg)];
			strcpy(userAgent, "Mozilla/");
			strcat(userAgent, arg);
		} else {
			//"Mozilla/4.5 [en] (X11; U; Linux 2.0.34 i486)"
			userAgent = "Mozilla/4.5 [en] (webfetch)";
		}
		wf->setUserAgent(userAgent);
		break;
#endif
	case 'n':
		wf->setContentType(arg);
		break;
	case 'o':
		if(strcmp("-", arg) == 0) {
			wf->setDataConsumer(&printData);
		} else {
			saveData.setOutputFile(arg);
			wf->setDataConsumer(&saveData);
		}
		break;
	case 'P':
		{
			HTTPProxyData *sp = wf->proxies.GetHTTPProxy();
			sp->setHostName(arg);
			if(verbose > 0 && sp->isActive()) {
				fprintf(stderr, "Using HTTP proxy %s:%d\n",
					sp->getHostName(),
					sp->getPort());
			}
		}
		break;
	case 'p':
		if(*arg == '@') {
			postData = new FileUploadData(&arg[1]);
		} else {
			postData = new StringUploadData(arg);
		}
		break;
	case 'R':
		wf->setFollowRedirect(1);
		break;
	case 'r':
		wf->setReferer(arg);
		break;
	case 'S':
		socksProxy = strdup(arg);
		break;
	case 's':
		// set up to save to file, but turn off previously set name
		saveData.setOutputFile(NULL);
		wf->setDataConsumer(&saveData);
		break;
	case 't':
		{
			long timeout = atol(arg);
			if(timeout <= 0) timeout = 60;
			wf->setTimeout(timeout);
		}
		break;
	case 'u':
		wf->setUserAgent(decodeUserAgent(arg));
		break;
	case 'U':
		wf->setUserAgent(arg);
		break;
	case 'v':
		verbose++;
		break;
	case 'V':
		printf("%s\n%s\n", wf->getUserAgent(), rcsid);
		exit(0);

#if HAVE_SSL > 0
	case 'k':
		if(strcasecmp(arg, "allow-expired") == 0) {
			ssloptions->setAllowExpired(1);
		} else if(strcasecmp(arg, "disallow-self-signed") == 0) {
			ssloptions->setAllowSelfSigned(0);
		} else if(strcasecmp(arg, "allow-self-signed") == 0) {
			ssloptions->setAllowSelfSigned(1);
		} else if(strcasecmp(arg, "disallow-untrusted") == 0) {
			ssloptions->setAllowUntrusted(0);
		} else if(strcasecmp(arg, "allow-untrusted") == 0) {
			ssloptions->setAllowUntrusted(1);
		} else if(strncasecmp(arg, "certfile=", 9) == 0) {
			ssloptions->setCertFile(&arg[9]);
		} else if(strncasecmp(arg, "certpath=", 9) == 0) {
			ssloptions->setCertPath(&arg[9]);
		} else if(strncasecmp(arg, "certkey=", 8) == 0) {
			ssloptions->setCertKeyFile(&arg[8]);
		} else {
			fprintf(stderr, "Unknown option -k %s\n", arg);
		}
		break;
#endif

	default:
		usage();
	}
	return 0;
}


static void doFile(Webfetch *wf, CookieJar *cookies, const char *file)
{
	FILE	*cmdFile;
	char	line[2048];

	cmdFile = fopen(file, "r");
	if(cmdFile == NULL) {
		fprintf(stderr, "cannot open '%s'\n", file);
		return;
	}
	while(fgets(line, sizeof(line), cmdFile) != NULL) {
		int l = strlen(line);
		// trim trailing white space
		while(l > 0 && isspace(line[l-1])) {
			l--;
			line[l] = '\0';
		}
		if(verbose > 0) printf("%s: %s\n", file, line);
		if(line[0] == '-') {
			l = 2;
			while(isspace(line[l])) l++;
			processArg(wf, cookies, line[1], &line[l]);
		} else {
			wf->processURL(httpCommand, line, postData);
		}
	}
	fclose(cmdFile);
}


class MyCookieJar : public CookieJar {
public:
	MyCookieJar();
protected:
	void handleCookie(const char *rawtext, const char *defdomain);
};

MyCookieJar::MyCookieJar() {}

void MyCookieJar::handleCookie(const char *rawtext, const char *defdomain)
{
	if(verbose > 0) fprintf(stderr, "Received Cookie: %s", rawtext);
	CookieJar::handleCookie(rawtext, defdomain);
}


int main(int argc, char *argv[])
{
	int	c;
	const char	*userAgent = "webfetch/"VERSION;

	wf = new Webfetch();
	wf->setUserAgent(userAgent);
	wf->setDataConsumer(&printData);

	CookieJar *cookieJar = new MyCookieJar();
	wf->setCookieJar(cookieJar);

#if HAVE_SSL > 0
	ssloptions = new SSLOptions();
	// ssloptions->setAllowSelfSigned(1);
	wf->setSSLOptions(ssloptions);
#endif

	while((c=getopt(argc, argv, "a:A:C:c:Ddef:H:hj:k:l:m:n:o:P:p:Rr:S:st:U:u:vV"))!=EOF) {
		int x;
		if((x = processArg(wf, cookieJar, c, optarg)) != 0) {
			exit(x);
		}
	}

	if(argc - optind < 1) {
		usage();
	}


	wf->setHeaderHandler(dealWithHeader);

	/* Decode options that need further work */

	// Look for proxy settings
	if(socksProxy != NULL) {
		SocksProxyData *sp = wf->proxies.GetSocksProxy();
		sp->setHostName(socksProxy);
		if(verbose > 0 && sp->isActive()) {
			fprintf(stderr, "Using SOCKS proxy %s:%d\n",
					sp->getHostName(),
					sp->getPort());
		}
	}

#if defined(unix)
	signal(SIGALRM, shutdown);
	signal(SIGTERM, shutdown);
	signal(SIGQUIT, shutdown);
	signal(SIGINT, shutdown);
#endif

#if defined(WIN32)
	/* we need the win socket initializers only when compiling on windows
	 * but as a console application.  In that case both
	 * WIN32 and CONSOLE_APP are defined.  However, it seems that in some
 	 * cases, if we use the win sock 2 libraries than we don't need this.
	 */
	WSADATA stWSAData;
	int nRet = WSAStartup(WINSOCK_VERSION, &stWSAData);
	/* WSAStartup() returns error value if failed (0 on success) */
	if (nRet != 0) {
		// WSAperror(nRet, "WSAStartup()", hInst);
		::fprintf(stderr, "Cannot initialize wsock. err=%d\n", nRet);
		exit(1);
	}
#endif

	/* get the urls */

#if PURGED_2002_07_22
	// check for old style usage:    webfetch host resource
	if(strncasecmp(argv[optind], "http:", 5) != 0 &&
	   argv[optind][0] != '@') {
		char *webSite = strdup(argv[optind]);
		optind++;
		for( ; optind < argc; optind++) {
			char *x = new char[20 + strlen(webSite) +
					strlen(argv[optind])];
			strcpy(x, "http://");
			strcat(x, webSite);
			strcat(x, "/");
			strcat(x, argv[optind]);
			wf->processURL(httpCommand, x, postData);
			delete [] x;
		}
		free(webSite);
		return 0;
	}
#endif

	for( ; optind < argc; optind++) {
		if(argv[optind][0] == '-') {
			int what = argv[optind][1];
			optind++;
			if(optind < argc) {
				processArg(wf, cookieJar, what, argv[optind]);
				optind++;
			}
		} else if(argv[optind][0] == '@') {
			doFile(wf, cookieJar, &argv[optind][1]);
		} else {
			wf->processURL(httpCommand, argv[optind],
						postData);
		}
	}

	int	status = wf->getErrorStatus();
	if(printStatus) {
		fprintf(stderr, "status: %d\n", status);
		fprintf(stderr, "http: %d\n", wf->getHTTPStatus());
	}
	delete cookieJar;
	return status;
}


