/* netio.h - deal with input from the network
 *
 * Copyright (c) 1997-2003 Tony Aiuto
 *
 * To report problems with this tool or to suggest enhancements send mail
 * to tony@aiu.to
 *
 */

/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * Edit History:
 *
 * $Log: netio.h $
 * Revision tools_TonyAiuto/3 2004/01/07 17:51:30 TonyAiuto
 * 	5.3.4 savepoint after HPUX and windows initial tweaks
 * 
 * Revision tools_TonyAiuto/2 2003/11/21 03:46:14 TonyAiuto
 * 	changes for 5.3.0
 * 
 */


#ifndef NETIO_H
#define NETIO_H 1


#include <stdio.h>

#if NOT_NEEDED
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include <stdlib.h>
#include <assert.h>

#include <sys/types.h>
#include <sys/stat.h>

// Bring in the platform specific indcludes

#if defined(unix)

#include <unistd.h>
#include <sys/socket.h>
#include <sys/utsname.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#elif defined(WIN32)

#include <winsock2.h>
#include <shlwapi.h>

#else
	no support for other OSes yet

#endif

#include <signal.h>
#if defined(WIN32)
void alarm(int x) { }
#define	SIGALRM	-1234
#endif

#include <webconstants.h>

#endif


class NetIOStream {
public:
	NetIOStream(int stream);
	virtual ~NetIOStream();

	int	nextByte();
	int	readLine(char *buf, int bufsize);
	int	read(char *buf, int bufsize);

	/* send should return # of bytes written, or an error code */
	virtual int	send(const char *, size_t);

	void	setTimeout(int);

	// how many bytes left in buffer
	int	left() { return  bufcount - idx; };

protected:
	int	ios;
	int	bufcount;	// how many bytes in buffer
	int	idx;		// index into where we are in buffer
	unsigned char	b[8192];

protected:
	virtual int	fill();

	int	timeout;
};


#if HAVE_SSL > 0
class SimpleSSL;

class NetIOStreamSSL : public NetIOStream {
public:
	NetIOStreamSSL(SimpleSSL *, int stream);
	~NetIOStreamSSL();

	int	send(const char *, size_t);
protected:
	int	fill();

private:
	SimpleSSL	*ssl;

};
#endif

#endif /* NETIO_H */
