/* proxy.cpp - classes to hold proxy server information
 *
 * Copyright (c) 1997-2002 Tony Aiuto
 *
 * To report problems with this tool or to suggest enhancements send mail
 * to tony@aiu.to
 *
 */

#ifndef lint
static char _copyright[] = "@(#) Copyright (c) 1997-2002, Tony Aiuto";
#endif

/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * Edit History:
 *
 * $Log: proxy.cpp $
 * Revision tools_TonyAiuto/5 2004/12/08 17:15:25 TonyAiuto
 * 	5.4.2 release - mostly portability changes for Solaris/macos/sco in
 * 	both the code and tests
 * 
 * Revision tools_TonyAiuto/4 2004/02/23 18:46:14 TonyAiuto
 * 	5.4 release
 * 
 * Revision tools_TonyAiuto/3 2003/10/26 03:15:18 TonyAiuto
 * 	5.2.1 - move placement of tobase64
 * 
 * Revision tools_TonyAiuto/2 2002/08/07 19:39:52 TonyAiuto
 * 	tweaks for windows
 * 
 */

#ifndef lint
static char rcsid[] = "@(#) $Id: proxy.cpp tools_TonyAiuto/5 2004/12/08 17:15:25 TonyAiuto $";
#endif


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>

// Bring in the platform specific indcludes


#if defined(WIN32)

#include <winsock2.h>
#include <shlwapi.h>

#else

#include <unistd.h>
void GetUserNameA(char *username, unsigned long *size);

#endif


#include "osdep.h"
#include "proxy.h"
#include "webconstants.h"
#include "webfetch.h"


ProxyData::ProxyData()
{
	active = false;
	hostName = NULL;
	port = -1;
	defaultPort = -1;
}

ProxyData::~ProxyData()
{
	setHostName(NULL);
}

void ProxyData::setHostName(const char * _hostname) 
{
	::free(hostName);

	if(_hostname == NULL || *_hostname == '\0') {
		hostName = NULL;
		active = false;
		port = -1;
	} else {
		hostName = strdup(_hostname);
		char *colon = strchr(hostName, ':');
		port = defaultPort;
		active = true;
		if(colon != NULL) {
			*colon = '\0';
			port = atoi(colon+1);
		}
	}
}

ProxyInfo::ProxyInfo()
{
	authorization = NULL;
}

ProxyInfo::~ProxyInfo()
{
	delete authorization;
}

void ProxyInfo::setAuthorization(const char *_auth)
{
	delete authorization;
	if(_auth == NULL || *_auth == '\0') {
		authorization = NULL;
	} else {
		authorization = Webfetch::tobase64(_auth);
	}
}

#if FUTURE
ostream& operator << ( ostream& os, ProxyData& pdata)
{
	os << "Proxy is " << (pdata.hostName) << "active, ";
	os << "Host: " << pdata.hostName << " Port: " << pdata.port;

	return os;
}

ostream& operator << ( ostream& os, ProxyInfo& pdata)
{
	os << endl << "    Socks Proxy " << endl << endl;
	if(pdata.m_socks_proxy.GetHasProxy())
		os << pdata.m_socks_proxy << endl;
	else
		os << "No Socks Proxy" << endl;

	os << endl << "    HTTP Proxy " << endl << endl;
	if(pdata.m_http_proxy.GetHasProxy())
		os << pdata.m_http_proxy << endl;
	else
		os << "No HTTP Proxy" << endl;
	return os;
}
#endif


HTTPProxyData::HTTPProxyData()
{
	defaultPort = DEFAULT_HTTP_PROXY_PORT;
}

HTTPProxyData::~HTTPProxyData()
{
}

bool HTTPProxyData::LoadDefaultProxyData()
{
	return false;
}


SocksProxyData::SocksProxyData()
{
	defaultPort = DEFAULT_SOCKS_PROXY_PORT;
}

SocksProxyData::~SocksProxyData()
{
}

bool SocksProxyData::LoadDefaultProxyData()
{
	return false;
}


#if !defined(WIN32)

void ProxyInfo::LoadSystemProxySettings()
{
	// TODO - in the future we could look at your netscape prefs
}


// windows emulation routines 

#include <pwd.h>
void GetUserNameA(char *username, unsigned long *size)
{
	struct passwd *pwent;

	pwent = getpwuid(geteuid());
	if(pwent == NULL) {
		strcpy(username, "me");
		*size = 2;
		return;
	}
	strncpy(username, pwent->pw_name, *size);
	*size = strlen(pwent->pw_name);
}

#endif /* unix */



#if defined(WIN32)


/*********************************************************************************
 *
 *  Windows proxy server lookup
 *
 *********************************************************************************/

//
// Constants
//

#define HTTP_PROXY_KEY "http="
#define SOCKS_PROXY_KEY "socks="

/* Systems running IE version: 5.00.2614.3500 report proxy as:
 * http://hostname:80, which is normal except for the leading http:// stuff so
 * we need to strip that junk out
 */
#define HTTP_HEADER "http://"

void ProxyInfo::LoadSystemProxySettings()
{
	HKEY key;
	bool proxy_enable;
	char *reg_path =
		"Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings";
	char *proxy_value = NULL;

	// First get the internet settings key

	LONG stat = ::RegOpenKeyEx(HKEY_CURRENT_USER, reg_path, 0,
							KEY_QUERY_VALUE, &key);

	if (stat != ERROR_SUCCESS) return;


	// See if any proxies are is enabled
	DWORD type;
	DWORD enable;
	DWORD size = sizeof(enable);

	stat = ::RegQueryValueEx(key, "ProxyEnable", 0, &type,
					(LPBYTE) &enable, &size);
	if (stat != ERROR_SUCCESS) {
		return;
	}

	proxy_enable = (enable != 0);
	if(!proxy_enable) {
		return;
	}

	size = 1000;
	proxy_value = (char *) ::malloc(size);
	stat = ::RegQueryValueEx(key, "ProxyServer", 0, &type,
					(LPBYTE) proxy_value, &size);
	if (stat == ERROR_MORE_DATA) {
		proxy_value = (char *) ::malloc(size);
		stat = ::RegQueryValueEx(key, "ProxyServer", 0, &type,
						(LPBYTE) proxy_value, &size);
	}
	if (stat != ERROR_SUCCESS) {
		if (proxy_value != NULL) free(proxy_value);
		return;
	}
	::RegCloseKey(key);


	// Now decode the value

	// This can either be:
	// type=host:port;type2=host2:port2;...
	//
	// Or, in the case of using the same proxy for everything EXCEPT SOCKS:
	//
	// Either -- <host>:<port>  Or  "http://<host>:<port>"
	// 
	// 

	char *start = proxy_value;
	char *next = NULL;

	char * equal = strchr(start, '=');
	char * semi = strchr(start, ';');
	if (equal == NULL && semi == NULL) {
		int len = strlen(HTTP_HEADER);

		// This can be an HTTP proxy, but not a socks proxy.

		if (strncmp(start, HTTP_HEADER, len) == 0) {
			start += len;
		}

		HTTPProxyData *sp = GetHTTPProxy();
		sp->setHostName(start);
	} else {
		do {
			next = strchr(start, ';');
			if (next != NULL) {
				*next++ = '\0';
			}

			int len = strlen(HTTP_PROXY_KEY);
			if (strncmp(HTTP_PROXY_KEY, start, len) == 0) {
				HTTPProxyData *sp = GetHTTPProxy();
				sp->setHostName(start+len);
			}

			len = strlen(SOCKS_PROXY_KEY);
			if (strncmp(SOCKS_PROXY_KEY, start, len) == 0) {
				SocksProxyData *sp = GetSocksProxy();
				sp->setHostName(start+len);
			}

			start = next;
		} while (next != NULL);
	}

	if (proxy_value != NULL) free(proxy_value);
}

/* get the version stamp of IE we are running
 * Caller frees the returned string
 * Not used yet.
 */
static char * RegistryGetIEVersion()
{
	char	version[100];
	HKEY	key;
	LONG stat = ::RegOpenKeyEx(HKEY_LOCAL_MACHINE,
				"SOFTWARE\\Microsoft\\Internet Explorer",
				0, KEY_QUERY_VALUE, &key);
	if(stat != ERROR_SUCCESS) return NULL;

	DWORD size = sizeof(version);
	DWORD type;
	stat = ::RegQueryValueEx(key, "Version", 0, &type,
					(LPBYTE) version, &size);
	if(stat != ERROR_SUCCESS) return NULL;

	::RegCloseKey(key);
	return ::strdup(version);
}

#endif /* WIN32 */
