/* proxy.h - classes that hold information about proxies in our way
 *
 */

/* Edit History:
 * $Log: proxy.h $
 * Revision tools_TonyAiuto/2 2004/01/07 17:51:30 TonyAiuto
 * 	5.3.4 savepoint after HPUX and windows initial tweaks
 * 
 */

#ifndef __proxy_h__
#define __proxy_h__

#if defined(hpux)
// This is to please some older HP-UX compilers
#define bool int
#define false 0
#define true 1
#endif


class ProxyData {
public:
	ProxyData();
	virtual ~ProxyData();

	virtual void setHostName(const char * hostname);
	void setActive(bool _active) { active = _active; }
	void setPort(int port) { port = port; }

	bool isActive() { return active; }
	char *getHostName() { return hostName; }
	int getPort() { return port; }

	// code must provide platform specific implementations of this
	virtual bool LoadDefaultProxyData() = 0;


protected:
	bool	active;
	char	*hostName;
	int	port;
	int	defaultPort;
};


class HTTPProxyData : public ProxyData {
public:
	HTTPProxyData();
	~HTTPProxyData();

	bool LoadDefaultProxyData();
};

class SocksProxyData : public ProxyData {
public:
	SocksProxyData();
	~SocksProxyData();

	bool LoadDefaultProxyData();
};

/* ProxyInfo holds information about our proxies.   It is
 * designed to hold info for multiple defined proxies, but
 * in reality, this is useless, because we can only use one.
 * Over time, I should probably factor the common stuff out
 * of the HTTP and Socks subclasses and just use a designator
 * to show which type it is.
 */

class ProxyInfo {
public:
	ProxyInfo();
	~ProxyInfo();

	void LoadSystemProxySettings();

	SocksProxyData * GetSocksProxy() { return &socksProxy; }
	HTTPProxyData * GetHTTPProxy() { return &httpProxy; }

	char *getAuthorization() { return authorization; }
	void setAuthorization(const char *);

private:
	SocksProxyData	socksProxy;
	HTTPProxyData	httpProxy;

	char	*authorization;		// Proxy authentication
};


#endif /* _proxy_h */
