/* ssloptions.cpp - options for SSL processing
 *
 * Copyright (c) 1997-2004 Tony Aiuto
 *
 * To report problems with this tool or to suggest enhancements send mail
 * to tony@aiu.to
 *
 */

#ifndef lint
static char _copyright[] = "@(#) Copyright (c) 1997-2004, Tony Aiuto";
#endif


/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdio.h>
#include <string.h>

#include "ssloptions.h"


SSLOptions::SSLOptions()
{
	certFile =  NULL;
	certPath =  NULL;
#if defined(DEFAULT_SSL_CERT_FILE)
	setCertFile(DEFAULT_SSL_CERT_FILE);
#endif
#if defined(DEFAULT_SSL_CERT_PATH)
	setCertPath(DEFAULT_SSL_CERT_PATH);
#endif
	certkeyFile = NULL;

	allowExpired = 0;
	allowSelfSigned = 0;
	allowUntrusted = 0;
}

SSLOptions::~SSLOptions()
{
	delete certFile;
	certFile = NULL;
	delete certPath;
	certPath = NULL;
	delete certkeyFile;
	certkeyFile = NULL;
}

void SSLOptions::setCertFile(const char *file)
{
	delete certFile;
	if(file == NULL || *file == '\0') {
		certFile = NULL;
	} else {
		certFile = new char [strlen(file) + 1];
		strcpy(certFile, file);
	}
}

void SSLOptions::setCertPath(const char *path)
{
	delete certPath;
	if(path == NULL || *path == '\0') {
		certPath = NULL;
	} else {
		certPath = new char [strlen(path) + 1];
		strcpy(certPath, path);
	}
}

void SSLOptions::setCertKeyFile(const char *file)
{
	delete certkeyFile;
	if(file == NULL || *file == '\0') {
		certkeyFile = NULL;
	} else {
		certkeyFile = new char [strlen(file) + 1];
		strcpy(certkeyFile, file);
	}
}

