#!/bin/sh

# proxy.sh
. ./testfuncs.sh

TEST_DEF bad socks proxy <<INP
$WEBFETCH -e -S snoblets http://tony.aiu.to/ 2>&1
echo prog: \$?
___EXPECT___
Unable to determine IP address for 'snoblets'
status: 4
http: 0
prog: 4
INP

TEST_DEF bad http proxy <<INP
$WEBFETCH -e -P snoblets http://tony.aiu.to/ 2>&1
echo prog: \$?
___EXPECT___
Unable to determine IP address for 'snoblets'
status: 5
http: 0
prog: 5
INP

TEST_DEF wrong service port for socks proxy <<INP
$WEBFETCH -e -S localhost:25 http://tony.aiu.to/ 2>&1
echo prog: \$?
echo a 404
___EXPECT___
Error talking to socks proxy.
status: 99
http: 0
prog: 99
a 404
INP

TEST_DEF successful http proxy <<INP
perl  tinyproxy.pl -p 8080 -l 1 &
sleep 1
$WEBFETCH -P localhost:8080 http://tony.aiu.to/robots.txt
echo prog: \$?
___EXPECT___
User-agent: *
Disallow: /bin/
Disallow: /family/
Disallow: /tools/
prog: 0
INP


TEST_DEF successful http proxy with proxy authentication <<INP
perl  tinyproxy.pl -p 8080 -l 1 -a proxy:lock &
sleep 1
$WEBFETCH -P localhost:8080 -A proxy:lock http://tony.aiu.to/robots.txt 2>&1
echo prog: \$?
___EXPECT___
User-agent: *
Disallow: /bin/
Disallow: /family/
Disallow: /tools/
prog: 0
INP

TEST_DEF missing http proxy authentication <<INP
perl  tinyproxy.pl -p 8080 -l 1 -a proxy:lock &
sleep 1
$WEBFETCH -e -P localhost:8080 http://tony.aiu.to/robots.txt 2>&1
echo prog: \$?
___EXPECT___
HTTP Status: 407 Proxy Authentication Required
status: 0
http: 407
prog: 0
INP


TEST_DEF failed http proxy authentication <<INP
perl  tinyproxy.pl -p 8080 -l 1 -a proxy:lock &
sleep 1
$WEBFETCH -e -P localhost:8080 -A proxy:bad http://tony.aiu.to/robots.txt 2>&1
echo prog: \$?
___EXPECT___
HTTP Status: 401 Unauthorized
Failed Proxy-Authorization
status: 0
http: 401
prog: 0
INP

# regular authorization
TEST_DEF bad server authorization through http proxy <<INP
perl  tinyproxy.pl -p 8080 -l 1 &
sleep 1
$WEBFETCH -P localhost:8080 -a foo:bar http://www.appletreegroup.com/protected/ 2>&1
___EXPECT___
HTTP Status: 401 Authorization Required
<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>401 Authorization Required</title>
</head><body>
<h1>Authorization Required</h1>
<p>This server could not verify that you
are authorized to access the document
requested.  Either you supplied the wrong
credentials (e.g., bad password), or your
browser doesn't understand how to supply
the credentials required.</p>
<hr />
<address>Apache/2.0.50 (FreeBSD) mod_ssl/2.0.50 OpenSSL/0.9.7d PHP/4.3.8 FrontPage/5.0.2.2635 Server at www.appletreegroup.com Port 80</address>
</body></html>
INP


TEST_DEF good server authorization through http proxy <<INP
perl  tinyproxy.pl -p 8080 -l 1 &
sleep 1
$WEBFETCH -P localhost:8080 -a guest:bar http://www.appletreegroup.com/protected/ 2>&1
___EXPECT___
<html>
<body>
nothing here
</body>
</html>
INP


TEST_DEF successful http proxy authenticaion with server authentication <<INP
perl  tinyproxy.pl -p 8080 -l 1 -a proxy:lock &
sleep 1
$WEBFETCH -P localhost:8080 -A proxy:lock -a guest:bar http://www.appletreegroup.com/protected/ 2>&1
echo prog: \$?
___EXPECT___
<html>
<body>
nothing here
</body>
</html>
prog: 0
INP


END_ALL_TESTS

exit 0

