#!/bin/sh

. ./testfuncs.sh

if ( $WEBFETCH https://www.cacert.org/ 2>&1 | grep Unsupported.scheme >$DEVNULL ) ; then
	echo No SSL support, SSL tests skipped >&3
	END_ALL_TESTS
fi

TEST_DEF selfcert1 self signed cert failure <<END_TEST
$WEBFETCH -e -o $DEVNULL https://buy.entrust.net/ 2>&1
echo prog: \$?
___EXPECT___
status: 10
http: 0
prog: 10
END_TEST


TEST_DEF selfcert2 self signed cert - allowed <<END_TEST
$WEBFETCH -e -o $DEVNULL -k allow-self-signed https://buy.entrust.net/ 2>.stderr
echo prog: \$?
cat .stderr
___EXPECT___
prog: 0
HTTP Status: 302 Moved Temporarily
Redirect to http://buy.entrust.net
status: 0
http: 302
END_TEST


TEST_DEF unkCA1 unknown CA failure <<END_TEST
$WEBFETCH -e -o $DEVNULL https://www.cacert.org/index.php 2>&1
___EXPECT___
status: 10
http: 0
END_TEST


TEST_DEF unkCA2 CA - allowed <<END_TEST
$WEBFETCH -e -o $DEVNULL -k allow-untrusted https://www.cacert.org/index.php 2>&1
___EXPECT___
status: 0
http: 200
END_TEST


TEST_DEF CAcert CA - CA cert provided <<END_TEST
$WEBFETCH -e -o $DEVNULL -k certfile=certs/cacert.crt https://www.cacert.org/index.php 2>&1
___EXPECT___
status: 0
http: 200
END_TEST


TEST_DEF CAdir CA - CA cert directory provided <<END_TEST
$WEBFETCH -e -o $DEVNULL -k certpath=certs https://www.cacert.org/index.php 2>&1
echo prog: \$?
___EXPECT___
status: 0
http: 200
prog: 0
END_TEST

END_ALL_TESTS
