# testfuncs.sh

OUT=/tmp/$$
NTESTS=0
NOK=0
NFAIL=0

DIFF="diff --ignore-space-change"
uName=`uname`
if [ $uName = SCO_SV ] ; then
	DIFF="diff -b"
fi
if [ $uName = SunOS ] ; then
	DIFF="/usr/bin/diff -b"
fi



# calling assumptions
# file 3 - things we want to print on the entire run stdout
# file 4 - status of the this test file

TEST_BEGIN()
{
	NTESTS=`expr $NTESTS + 1`
	if [ "$VERBOSE" -gt 0 ] ; then
		echo "TEST $*" >&3
	fi
	echo TEST "$*"
}

END_ALL_TESTS()
{
	echo $NTESTS tests, $NOK passed, $NFAIL failed >&4
	# echo exit $NFAIL >&3
	if [ $NFAIL -gt 0 ] ; then
		echo $NFAIL >.failed
	else
		$RM -f .failed
	fi
	exit $NFAIL
}


TEST_DEF()
{
	name="$1"
	shift
	desc="$*"

	NTESTS=`expr $NTESTS + 1`
	if [ "$VERBOSE" -gt 1 ] ; then
		echo "TEST $name $desc" >&3
	fi

	cmd="/tmp/$$.cmd"
	expected="/tmp/expected.$name.$$"
	got="/tmp/got.$name.$$"
	TFILE=/tmp/$$.diff
	trap "$RM -f $cmd $expected $got $TFILE; exit 0" 0 1 2 3 15
	# start appending to cmd
	file=$cmd
	while read line ; do
		if [ "$line" != '___EXPECT___' ] ; then
			echo "$line" >>$file
		else
			# swap so we write to expected
			file="$expected"
		fi
	done
	if [ "$VERBOSE" -gt 2 ] ; then
		echo 'Command {' >&3
		cat $cmd >&3
		echo '}' >&3
	fi	
	/bin/sh $cmd >"$got"
	$DIFF -c "$expected" "$got" | grep -v 'No differences encountered' >$TFILE
	if [ -s $TFILE ] ; then
		NFAIL=`expr $NFAIL + 1`
		if [ "$VERBOSE" -gt 0 ] ; then
			printf '%-20s failed\n' "$name:" >&3
		fi
		cat $TFILE >&3
		err=1
	else
		NOK=`expr $NOK + 1`
		if [ "$VERBOSE" -gt 0 ] ; then
			printf '%-20s passed\n' "$name:" >&3
		fi
	fi
	$RM -f "$cmd" "$expected" "$got" $TFILE
	return $err
}
