/* upload.cpp - interface to data we want to upload to the server
 *
 * Copyright (c) 1997-2003 Tony Aiuto
 *
 * To report problems with this tool or to suggest enhancements send mail
 * to tony@aiu.to
 *
 */

#ifndef lint
static char _copyright[] = "@(#) Copyright (c) 1997-2002, Tony Aiuto";
#endif

/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * Edit History:
 *
 * $Log: upload.cpp $
 * Revision tools_TonyAiuto/5 2004/12/08 17:15:25 TonyAiuto
 * 	5.4.2 release - mostly portability changes for Solaris/macos/sco in
 * 	both the code and tests
 * 
 * Revision tools_TonyAiuto/4 2004/05/28 15:36:24 TonyAiuto
 * 	5.4.1 checkpoint
 * 
 * Revision tools_TonyAiuto/3 2004/02/23 18:46:14 TonyAiuto
 * 	5.4 release
 * 
 */

#ifndef lint
static char rcsid[] = "@(#) $Id: upload.cpp tools_TonyAiuto/5 2004/12/08 17:15:25 TonyAiuto $";
#endif


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>

#include <stdlib.h>
#include <assert.h>

#include <sys/types.h>
#include <sys/stat.h>

// Bring in the platform specific indcludes

#if defined(WIN32)

#include <winsock2.h>
#include <shlwapi.h>

#define PATH_SEP_CHAR '\\'

#else

#include <unistd.h>

#endif

#include "osdep.h"
#include <webfetch.h>
#include <webconstants.h>
#include <netio.h>


UploadData::UploadData() { }

UploadData::~UploadData() { }

const char *UploadData::getContentType() const
{
	return "application/x-www-form-urlencoded";
}



StringUploadData::StringUploadData(const char *data)
{
	sdata = new char [strlen(data) + 1];
	strcpy(sdata, data);
}

StringUploadData::~StringUploadData()
{
	delete sdata;
}

int StringUploadData::getSize() const
{
	if(sdata != NULL) return strlen(sdata);
	return 0;
}

int StringUploadData::send(NetIOStream *ios) const
{
	if(sdata != NULL) {
		return ios->send(sdata, strlen(sdata));
	}
	return 0;
}

FileUploadData::FileUploadData(const char *filename)
{
	file = fopen(filename, "rb");
}

FileUploadData::~FileUploadData()
{
	if(file != NULL) fclose(file);
}

int FileUploadData::getSize() const
{
	struct stat sbuf;
	if(file == NULL) return 0;
	int err = fstat(fileno(file), &sbuf);
	if(err == 0) {
		return sbuf.st_size;
	}
	return 0;
}


const char *FileUploadData::getContentType() const
{
	// There is room for opinion about this choice
	return "application/x-www-form-urlencoded";
}


int FileUploadData::send(NetIOStream *ios) const
{
	char	buf[4096];
	int	err = 0;

	size_t left = this->getSize();

	while(left > 0) {
		size_t	got;

		size_t nread = sizeof(buf);
		if(left < nread) nread = left;

		got = fread(buf, 1, nread, file);
		// printf("got %d bytes from read file to upload\n", got);
		if(got < 0) return got;

		left -= got;
		if(got > 0) {
			// printf("Sending POST: %s###\n", buf);
			// printf("sending %d bytes over net\n");
			err = ios->send(buf, got);
			// printf("FileUpload: got err=%d\n", err);
			if(err < 0) return err;
		}
	}
	return err;
}



/* WF_MPComponent is a helper classs to MultipartUploadData.
 * It holds an individual component and threads them into
 * a list.
 * name:	what the element is called.  This corresponds to the
 *		'name' attribute of a form INPUT tag
 * value:	This corresponds to what the user entered in a form
 *		field.   So, for '<input type=FILE>', it is the filename
 *		rather than the contents of the file.
 */

class WF_MPComponent {
public:
	WF_MPComponent(const char *name, const char *value);
	~WF_MPComponent();

	int getSize() const;
	int send(NetIOStream *) const;

	char	*name;		// the name of the form file
	char	*value;		// and the value
	UploadData *elem;

private:
	WF_MPComponent *next;
	int		isFile;

	friend class MultipartUploadData;
};
static const char *CD = "Content-Disposition: form-data; name=";


WF_MPComponent::WF_MPComponent(const char *_name, const char *_value)
{
	name = new char [strlen(_name) + 1];
	strcpy(name, _name);
	isFile = 0;
	next = NULL;

	if(*_value == '@') {
		_value++;
		elem = new FileUploadData(_value);
		isFile = 1;
	} else {
		elem = new StringUploadData(_value);
	}
	value = new char [strlen(_value) + 1];
	strcpy(value, _value);
}


WF_MPComponent::~WF_MPComponent()
{
	delete elem;
	delete next;
}



/* getSize of a component has to yield the total number of bytes
 * including all the header crap
 */

int WF_MPComponent::getSize() const
{
/* SAMPLE
Content-Disposition: form-data; name="pmIPNO"

hello
-----------------------------7198853861649760492596516649
Content-Disposition: form-data; name="FILE"; filename="ZZZ"
Content-Type: application/octet-stream

*/

	int ret = strlen(CD) + strlen(name) + 3;   /* cd + " + name + " + NL */
	if(isFile) {
		ret += 13 + strlen(value);	/* >>>; filename="XXX"<<< */
	}
	ret += 14 + strlen(elem->getContentType()) + 1;	/* Content-Type: + ? + NL */

	ret += 1;				/* NL as header delimeter */
	ret += elem->getSize();			/* body of data */
	ret += 1;				/* a final traling NL */
	return ret;
}


int WF_MPComponent::send(NetIOStream *ios) const
{
	int err = 0;
	err = ios->send(CD, strlen(CD));
	if(err < 0) return err;
	err = ios->send(name, strlen(name));
	if(err < 0) return err;
	err = ios->send("\"", 1);
	if(err < 0) return err;
	if(isFile) {
		err = ios->send("; filename=\"", 12);
		if(err < 0) return err;
		err = ios->send(value, strlen(value));
		if(err < 0) return err;
		err = ios->send("\"", 1);
		if(err < 0) return err;
	}
	err = ios->send("\n", 1);
	if(err < 0) return err;

#if TODO
-Disposition: form-data; name="FILE"; filename="ZZZ"
	Content-Type: application/octet-stream
		return ios->send(sdata, strlen(sdata));
#endif
	return err;
}


MultipartUploadData::MultipartUploadData(const char *data)
{
	parts = NULL;
	boundary = new char[100];
	sprintf(boundary, "-----------------------------%08x%3x",
			this, strlen(data));
	sdata = new char [strlen(data) + 1];
	strcpy(sdata, data);
}

MultipartUploadData::~MultipartUploadData()
{
	delete sdata;
	delete boundary;
	delete parts;
}

int MultipartUploadData::getSize() const
{
	WF_MPComponent *p;
	int	ret = 0;
	for(p = parts; p != NULL; p = p->next) {
		ret += p->getSize();
		ret += strlen(boundary) + 1;
	}
	return 0;
}

int MultipartUploadData::send(NetIOStream * /* ios */) const
{
	WF_MPComponent *p;
	int	ret = 0;
	for(p = parts; p != NULL; p = p->next) {
	}
	return 0;
}

const char *MultipartUploadData::getContentType() const
{
	return "multipart/form-data";
}

/* SAMPLE
<form method="post" action="http://foo.com/scripts/upload.asp"
enctype="multipart/form-data">
<input type="hidden" name="UPLOAD_REQ_NO" value="00000074">
<input type="hidden" name="pmIPNO"        value="0001">
<input type="hidden" name="pmPROFILE"     value="1">
<input type="file"   name="FILE"          size="40">
<input type="submit"                      value="Finish">
</form>
*/
