/* webfetch.cpp - fetch a file from an http server
 *
 * Copyright (c) 1997-2002 Tony Aiuto
 *
 * To report problems with this tool or to suggest enhancements send mail
 * to tony@aiu.to
 *
 */

#ifndef lint
static char _copyright[] = "@(#) Copyright (c) 1997-2002, Tony Aiuto";
#endif

/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * Edit History:
 *
 * $Log: webfetch.cpp $
 * Revision tools_TonyAiuto/36 2004/12/08 17:15:25 TonyAiuto
 * 	5.4.2 release - mostly portability changes for Solaris/macos/sco in
 * 	both the code and tests
 * 
 * Revision tools_TonyAiuto/35 2004/05/28 15:36:24 TonyAiuto
 * 	5.4.1 checkpoint
 * 
 * Revision tools_TonyAiuto/34 2004/02/23 18:46:14 TonyAiuto
 * 	5.4 release
 * 
 * Revision tools_TonyAiuto/33 2004/01/07 17:51:30 TonyAiuto
 * 	5.3.4 savepoint after HPUX and windows initial tweaks
 * 
 * Revision tools_TonyAiuto/32 2003/11/21 03:46:14 TonyAiuto
 * 	changes for 5.3.0
 * 
 * Revision tools_TonyAiuto/31 2003/11/15 01:36:23 TonyAiuto
 * 	SSL working, but cert verification flakey
 * 
 * Revision tools_TonyAiuto/30 2003/11/13 22:08:08 TonyAiuto
 * 	5.3.0 checkpoint
 * 
 * Revision tools_TonyAiuto/29 2003/11/13 20:30:19 TonyAiuto
 * 	interim checkpoint
 * 
 * Revision tools_TonyAiuto/28 2003/10/26 03:15:18 TonyAiuto
 * 	5.2.1 - move placement of tobase64
 * 
 * Revision tools_TonyAiuto/26 2003/08/21 13:33:39 TonyAiuto
 * 	fix bug where "-o file -C head" would not work, output was always to stdout.
 * 
 * Revision tools_TonyAiuto/25 2003/01/17 17:10:44 TonyAiuto
 * 	fix bug with basic authorization
 * 
 * Revision tools_TonyAiuto/24 2002/10/17 19:16:17 TonyAiuto
 * 	fix version
 * 
 * Revision tools_TonyAiuto/23 2002/10/17 18:08:22 TonyAiuto
 * 	5.1.0 release
 * 
 * Revision tools_TonyAiuto/22 2002/08/07 19:58:25 TonyAiuto
 * 	5.0.0 beta
 * 
 * Revision tools_TonyAiuto/21 2002/07/18 16:01:36 TonyAiuto
 * 	initial 5.0.0 work
 * 
 */

#ifndef lint
static char rcsid[] = "@(#) $Id: webfetch.cpp tools_TonyAiuto/36 2004/12/08 17:15:25 TonyAiuto $";
#endif

#define	WEBFETCH_VERSION "5.1.1"


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include <stdlib.h>
#include <assert.h>

#include <sys/types.h>
#include <sys/stat.h>

// Bring in the platform specific indcludes


#if defined(WIN32)

#include <winsock2.h>
#include <shlwapi.h>

#define PATH_SEP_CHAR '\\'

#else

#include <unistd.h>
#include <sys/socket.h>
#include <sys/utsname.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>

#define PATH_SEP_CHAR '/'
void GetUserNameA(char *username, unsigned long *size);

#endif

#include <signal.h>
#if defined(WIN32)
void alarm(int x) { }
#define	SIGALRM	-1234
#endif

#include "osdep.h"
#include <webfetch.h>
#include <webconstants.h>
#if HAVE_SSL > 0
#include <sslfuncs.h>
#endif
#include <netio.h>

extern	int	verbose;



void Webfetch::processURL(int httpCommand, const char *_url,
				const UploadData *postArgs)
{
	errorStatus = 0;
	httpStatus = 0;
	Protocol_Handler *handler = NULL;
	current_handler = NULL;

	// skip "scheme:"
	const char *sp = _url;
	while(*sp && *sp != ':') sp++;
	if(*sp != ':') {
		fprintf(stderr, "Missing scheme (%s)\n", _url);
		errorStatus = UNSUPPORTED_SCHEME;
		return;
	}
	sp++;
	while(*sp == '/') sp++;

	char *webSite = strdup(sp);	// have a copy of "host:port/resource"
	int port = -1;			// reset to default
	char *t = webSite;
	while(*t && *t != '/') {
		if(*t == ':') {
			*t++ = '\0';	// split to host + port
			port = atoi(t);
		}
		t++;
	}

	char *url = NULL;
	if(*t == '\0') {
		url = strdup("/");	// we had 'http://host' w/o trailing /
	} else {
		url = strdup(t);	// t points to / between host/resource
	}
	*t = '\0';		// terminate host

	assert(*url == '/');

	if(strncasecmp(_url, "http:", 5) == 0) {
		handler = new HTTP_Handler(this, webSite, port);
	}
#if HAVE_SSL
	if(strncasecmp(_url, "https:", 6) == 0) {
		handler = new HTTPS_Handler(this, webSite, port);
	}
#endif
	// TODO - add new schemes here

	if(handler == NULL) {
		fprintf(stderr, "Unsupported scheme %s\n", _url);
		errorStatus = UNSUPPORTED_SCHEME;
		free((char *) webSite);
		return;
	}
	current_handler = handler;

	// Try to connect
	alarm(timeout);
	const char *physicalServer = webSite;
	if(connectHost != NULL) physicalServer = connectHost;
	int err = handler->TCPconnect(physicalServer);
	alarm(0);			// cancel connect timeout

	if(err != 0) {
		errorStatus = err;
	} else {
		err = handler->getNetIOStream();
		if(err != 0) {
			// if we got a positive err, override the current status
			if(err > 0) errorStatus = err;
			else if(errorStatus == 0) {
				errorStatus = OTHER_FAILURE;
			}
		} else {

			// We finally connected, now do the stuff

			handler->doit(httpCommand, url, postArgs);
			handler->shutdown();
			httpStatus = handler->httpStatus;
		}
	}

	delete handler;
	free(url);
	return;
}




/* Resolve a host name to an IP address
 */

static int hostnameToIP(const char *hostname, int port,
				struct sockaddr_in *ipstruct)
{
	if (strlen(hostname) > 255) {
		fprintf(stderr, "host name too long.\n");
		return Webfetch::BAD_HOSTNAME;
	}
	// printf("hostnameToIP(%s, %d)\n", hostname, port);

	memset(ipstruct, 0, sizeof *ipstruct);

#ifdef BROKEN_INET_ADDR
	ipstruct->sin_addr.s_addr = inet_network(hostname);
#else /* BROKEN_INET_ADDR */
	ipstruct->sin_addr.s_addr = inet_addr(hostname);
#endif /* BROKEN_INET_ADDR */
	if ((ipstruct->sin_addr.s_addr & 0xffffffff) != 0xffffffff) {
		/* It was a valid numeric host address. */
		ipstruct->sin_family = AF_INET;
	} else {
		/* Look up the host address from the name servers. */
		struct hostent *hp = gethostbyname(hostname);
		if (hp == NULL || hp->h_addr_list[0] == NULL) {
			fprintf(stderr,
				"Unable to determine IP address for '%.100s'\n",
				hostname);
			return Webfetch::UNKNOWN_HOST;
		}

		ipstruct->sin_family = hp->h_addrtype;
		memcpy(&ipstruct->sin_addr, hp->h_addr_list[0],
				sizeof ipstruct->sin_addr);
	}
	ipstruct->sin_port = htons(port);

	return 0;
}

/* Figure out what host to connect to, taking proxy settings into account
 * @param	host		final host name we really want
 * @param	port		final port we really want
 */

int Webfetch::resolveHosts(const char *host, int port,
			struct sockaddr_in *hostIP,
			struct sockaddr_in *connectIP)
{

	int	err = 0;
	SocksProxyData *socks = proxies.GetSocksProxy();
	HTTPProxyData *http = proxies.GetHTTPProxy();


	/* first try socks proxy */
	if(socks->getHostName() != NULL) {
		// IP of host must resolve or we are stuck
		err = hostnameToIP(host, port, hostIP);
		if(err != 0) {
			return err;
		}
		err = hostnameToIP(socks->getHostName(),socks->getPort(),connectIP);
		if(err != 0) {
			socks->setActive(false);
			return BAD_SOCKS_PROXY_HOST;
		}
	}

	/* if we don't have a valid socks proxy, try http proxy */
	if((!socks->isActive()) && (http->getHostName() != NULL)) {
		err = hostnameToIP(http->getHostName(), http->getPort(), connectIP);
		if(err != 0) {
			http->setActive(false);
			return BAD_HTTP_PROXY_HOST;
		}
	}

	if(!socks->isActive() && !http->isActive()) {
		err = hostnameToIP(host, port, hostIP);
		*connectIP = *hostIP;
	}
	return err;
}


/* Deal with SOCKS 4 proxy connection startup
 */
int Webfetch::negotiateWithSocks4Proxy(int proxySocket, struct sockaddr_in *hostIP)
{
	char * username = NULL;
	char username_buf[512];
	unsigned char data[1024];
	int	err = 0;

	SocksProxyData *socks = proxies.GetSocksProxy();

	username = username_buf;
	unsigned long size = sizeof(username_buf);
	::GetUserNameA(username, &size);
	username[size] = '\0';

	/* Build the socks4 connect packet */

	unsigned char *ptr = data;
	*ptr++ = 4;	// Socks 4;
	*ptr++ = 1;	// A Connect request;
	*((short *) ptr) = hostIP->sin_port; // Send 2 bytes for the port.
	ptr += 2;
#if defined(WIN32)
	*((long *) ptr) = hostIP->sin_addr.S_un.S_addr; // Send 4 byte IP address.
	ptr += 4;
#else
	*ptr++ = (unsigned char) (hostIP->sin_addr.s_addr) & 0xff;
	*ptr++ = (unsigned char) (hostIP->sin_addr.s_addr >> 8) & 0xff;
	*ptr++ = (unsigned char) (hostIP->sin_addr.s_addr >> 16) & 0xff;
	*ptr++ = (unsigned char) (hostIP->sin_addr.s_addr >> 24) & 0xff;
#endif
	strcpy((char *) ptr, username); // The variable length user name.
	ptr += strlen(username);
	*ptr++ = 0;

	// Send it over ...
	::send(proxySocket, (char *)data, ptr - data, 0);

	// ... look for an 8 byte response

	int got = 0, want = 8;
	char * start = (char *) data;
	while (got < want) {
		int need = want - got;
		int count = ::recvfrom(proxySocket, start, need, 0, NULL, NULL);

		if (count < 0) {
			err = SOCKS_BAD_DATA;
			break;
		} else {
			start += count;
			got += count;
		}
	}

	if (err != 0) return err;

	if (got == 8) {
		if (data[0] != 0) {
			err = SOCKS_BAD_REPLY_CODE;
		} else if (data[1] != SOCKS_SUCCESS) {
			err = (int) data[1];
		}
	} else {
		err = SOCKS_BAD_DATA_LENGTH;
	}

	if (err != 0) {
		fprintf(stderr, "Error talking to socks proxy.\n");
		//err->AddArgument(port);
		//err->AddArgument(hostname);
		//if (username != NULL)
		//err->AddArgument(username);
	}
	if(err > 0) err = -err;
	return err;
}

static char *printIP(long ip)
{
	static char tmp[40];

#if defined(BYTE_ORDER) && BYTE_ORDER == 4321
	sprintf(tmp, "%d.", (ip >> 24) & 0xff);
	sprintf(&tmp[strlen(tmp)], "%d.", (ip >> 16) & 0xff);
	sprintf(&tmp[strlen(tmp)], "%d.", (ip >> 8) & 0xff);
	sprintf(&tmp[strlen(tmp)], "%d", ip & 0xff);
#else
	/* __LITTLE_ENDIAN 1234 */
	sprintf(tmp, "%d.", ip & 0xff);
	sprintf(&tmp[strlen(tmp)], "%d.", (ip >> 8) & 0xff);
	sprintf(&tmp[strlen(tmp)], "%d.", (ip >> 16) & 0xff);
	sprintf(&tmp[strlen(tmp)], "%d", (ip >> 24) & 0xff);
#endif
	return tmp;
}



static char *fixname(const char *fname)
{
	const char *x;
	char *ret = new char[strlen(fname) + 1];
	char *r;

	r = ret;
	for(x = fname; *x;) {
		char c = *x++;
		if(c == '%') {
			char d = *x++;
			if(d >= 'A')	c = d - 'A' + 10;
			else		c = d - '0';
			c <<= 4;
			d = *x++;
			if(d >= 'A')	c |= d - 'A' + 10;
			else		c |= d - '0';
		}
		if(c == '!' || c == '~') {
			c = '_';
#if CONVERT_SPACE_TO_UNDERSCORE
		} else if(c == ' ') {
			c = '_';
#endif
		} else if(c == '\'') {
			continue;
		}
		*r++ = c;
	}
	*r = '\0';

	// printf("Fix: %s -> %s\n", fname, ret);
	return ret;
}



Webfetch::Webfetch()
{

	/* Get any default proxy settings we can.   After thay we decode the
	 * command line settings, so we can override them.   Use -P "" to
	 * make sure they system proxy settting is not used
	 */
	proxies.LoadSystemProxySettings();

	cookieJar = NULL;
	authorization = NULL;
	cookie = NULL;
	connectHost = NULL;
	followRedirect = 0;
	from = NULL;
	language = NULL;       // content of Language header
	referer = NULL;
	contentType = NULL;
	timeout = 60;
	userAgent = strdup("webfetch/"WEBFETCH_VERSION);
	virtualHost = NULL;
}


Webfetch::~Webfetch()
{
	if(authorization != NULL) delete authorization;
	if(cookie != NULL) free(cookie);
	if(connectHost != NULL) free(connectHost);
	if(language != NULL) free(language);
	if(referer != NULL) free(referer);
	if(userAgent != NULL) free(userAgent);
	if(virtualHost != NULL) free(virtualHost);
}


void Webfetch::shutdown()
{
	if(current_handler != NULL) {
		current_handler->shutdown();
		current_handler = NULL;
	}
	consumer->endStream();

}

void Webfetch::setHeaderHandler(HeaderHandler *hh)
{
	headerHandler = hh;
}


/* Generic header handler, subclasses may override this
/* Subclasses which override handleHeader must call the
 * protocol's processHeader to handle default things
 * The sequence should look like:
 *
 *	MyClass::handleHeader(const char *x, int n) {
 *		if(headerHandler != NULL) {
 *			headerHandler(this, x, n);
 *		}
 *		handler->processHeader(x, n);
 *
 *		// my special header processing
 *	}
 */

void Webfetch::handleHeader(const char *x, int n)
{
	// give application header handler the right to process it
	if(headerHandler != NULL) {
		headerHandler(this, x, n);
	}
	current_handler->processHeader(x, n);
}




void Webfetch::setAuthorization(const char *x)
{
	if(authorization != NULL) delete authorization;
	authorization = tobase64(x);
}

void Webfetch::setCookie(const char *x)
{
	if(cookie != NULL) free(cookie);
	cookie = strdup(x);
}

void Webfetch::setCookieJar(CookieJar *newJar)
{
	cookieJar = newJar;
}

void Webfetch::setConnectHost(const char *hostname)
{
	if(connectHost != NULL) free(connectHost);
	connectHost  = strdup(hostname);
}

void Webfetch::setContentType(const char *_ctype)
{
	if(contentType != NULL) free(contentType);
	if(_ctype == NULL) {
		contentType  = NULL;
	} else {
		contentType  = strdup(_ctype);
	}
}

void Webfetch::setDataConsumer(DataConsumer *x)
{
	consumer = x;
}

void Webfetch::setFollowRedirect(int x)
{
	followRedirect = x;
}

void Webfetch::setFrom(const char *x)
{
	if(from != NULL) free(from);
	from = strdup(x);
}

void Webfetch::setLanguage(const char *x)
{
	if(language != NULL) free(language);
	language = strdup(x);
}

void Webfetch::setReferer(const char *x)
{
	if(referer != NULL) free(referer);
	referer = strdup(x);
}

void Webfetch::setTimeout(long _timeout)
{
	timeout = _timeout;
}

void Webfetch::setUserAgent(const char *x)
{
	if(userAgent != NULL) free(userAgent);
	userAgent = strdup(x);
}

void Webfetch::setVirtualHost(const char *hostname)
{
	if(virtualHost != NULL) free(virtualHost);
	virtualHost  = strdup(hostname);
}

#if HAVE_SSL > 0
/** setSSLOptions - associates SSL options with this Webfetch instance.
 * The caller still owns the data and is expected to not destroy
 * the SSLOptions instance until after the Webfetch instance is destroyed.
 */
void Webfetch::setSSLOptions(SSLOptions *ssloptions)
{
	SimpleSSL::setOptions(ssloptions);
}
#endif

/* utility function to translate a string to base64 encoding
 * 
 * @arg	s	string to convert
 * @return	converted string, caller must delete
 */

char *Webfetch::tobase64(const char *_s)
{
	const unsigned char *s = (const unsigned char *) _s;
	char *ret = new char[strlen(_s) * 2 + 1];	// too much!
	char *o = ret;
	const char *encode = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	long word;
	// printf("tobase64(%s) ", _s);
	while(*s) {
		int	isize = 1;
		// Pick up 3 octets to make a 24 bit field
		word = *s++;
		word <<= 8;
		if(*s) { word |= *s++; isize++; }
		word <<= 8;
		if(*s) { word |= *s++; isize++; }

		// emit as 4 six bit octets
		// printf("\t%08x ", word);
		// printf("%c", encode[(word >> 18) & 0x3f]);
		*o++ = encode[(word >> 18) & 0x3f];
		// printf("%c", encode[(word >> 12) & 0x3f]);
		*o++ = encode[(word >> 12) & 0x3f];
		if(isize == 1) {
			*o++ = '=';
			*o++ = '=';
		} else {
			// printf("%c", encode[(word >> 6) & 0x3f]);
			*o++ = encode[(word >>  6) & 0x3f];
			if(isize == 2) {
				*o++ = '=';
			} else {
				// printf("%c", encode[(word ) & 0x3f]);
				*o++ = encode[(word      ) & 0x3f];
			}
		}
		// printf("\n");
	}
	*o = '\0';
	// printf("-> '%s'\n", ret);
	return ret;
}


/** DataConsumer
 */


DataConsumer::DataConsumer() { }
DataConsumer::~DataConsumer() {}

PrintingDataConsumer::PrintingDataConsumer() { /* printf("PDC::ctor\n"); */ }
PrintingDataConsumer::~PrintingDataConsumer() { /* printf("PDC::dtor\n"); */ }

void PrintingDataConsumer::startStream(const char * /* url */) { }
void PrintingDataConsumer::takeData(const char *buf, int len)
{
	fwrite(buf, 1, len, stdout);
	fflush(stdout);
}

void PrintingDataConsumer::endStream()
{
	fflush(stdout);
}


FileDataConsumer::FileDataConsumer()
{
	createDir = 0;
	outputFileName = NULL;
	ofile = NULL;
}
FileDataConsumer::~FileDataConsumer()
{
	if(outputFileName != NULL) free(outputFileName);
	if(ofile != NULL) fclose(ofile);
}


void FileDataConsumer::takeData(const char *buf, int len)
{
	if(ofile != NULL) {
		fwrite(buf, 1, len, ofile);
		fflush(ofile);
	}
}

void FileDataConsumer::endStream()
{
	if(ofile != NULL) fclose(ofile);
	ofile = NULL;
}

void FileDataConsumer::setCreateDirectories(int x)
{
	createDir = x;
}

void FileDataConsumer::setOutputFile(const char *_file)
{
	this->endStream();
	if(outputFileName != NULL) free(outputFileName);
	if(_file != NULL && *_file != '\0') {
		outputFileName = strdup(_file);
	} else {
		outputFileName = NULL;
	}
}


void FileDataConsumer::startStream(const char *resource)
{
	const char	*fname = NULL;

	/* Get the output stream to write results on */

	const char	*p;	// tmp ptr into full output file name
	// Write to a file, either we were given the name or we find it
	if(outputFileName) {
		p = fixname(outputFileName);	// convert %xx escapes
	} else {
		/* Build a file name from the resource name */
		p = fixname(resource);
		while(*p == '/') p++;		// no leading /s
	}

	// strip leading ./
	while(strncmp(p, "./", 2) == 0) p += 2;

	fname = strrchr(p, '/');	// find basename

	if(fname == NULL) {
		fname = p;		// no /, basename is file
	} else {
		fname++;		// found /, basename follows
	}
	if(createDir && p != fname) {	// Build directory part 
		char	dir[512];
		char	*d, *dpart;

		// Make a copy of the resource, we trash it
		strcpy(dir, p);

		d = dir;
		while(dpart = strchr(d, '/')) {
			*dpart = '\0';
#if defined(WIN32)
			CreateDirectory(dir, NULL);
#else
			mkdir(dir, 0755);
#endif
			*dpart = PATH_SEP_CHAR;
			d = dpart + 1;
		}
		// Use whole file name
		fname = p;
	}
	if(outputFileName != NULL) {
		// for outfile we always use the designated file
		fname = p;
	}
	if(verbose > 0) printf("writing to '%s'\n", fname);
	ofile = fopen(fname, "wb");
	if(ofile == NULL) {
		fprintf(stderr, "Cannot open '%s' for writing.\n", fname);
		return;
	}
}


/** Protocol Handlers
 */

Protocol_Handler::Protocol_Handler(Webfetch *_context, const char *_host, int _port ) :
	context(_context),  port(_port)
{
	host = strdup(_host);
	serverFD = -1;
}

Protocol_Handler::~Protocol_Handler()
{
	shutdown();
	context = NULL;
	free(host);
}

/* Set up the connection to the server, possibly through a proxy
 *
 * host may be of the form host:port.
 * host		hostname to connect to (with optional :port)
 * @return	error code or 0 for success
 *
 * Side effects:
 * may change port 
 * sets serverFD with file descripter if connect succeeds
 */

int Protocol_Handler::TCPconnect(const char *_host)
{
	struct sockaddr_in hostIP;
	struct sockaddr_in connectIP;
	int	sock;

	serverFD = -1;
	char *host = strdup(_host);
	// look for :port on web host
	char *colon = strchr(host, ':');
	if(colon != NULL) {
		*colon = '\0';
		port = atoi(colon+1);
	}

	int err = context->resolveHosts(host, port, &hostIP, &connectIP);
	if(err != 0) {
		free(host);
		return err;
	}
	if(verbose > 1) {
		printf("open TCP connection to '%s', port = %d\n",
			printIP(connectIP.sin_addr.s_addr),
			ntohs(connectIP.sin_port));
	}

	/* Create the socket. */
	sock = socket(connectIP.sin_family, SOCK_STREAM, 0);
	if (sock < 0) {
		fprintf(stderr, "socket: %.100s\n", strerror(errno));
		free(host);
		return Webfetch::OTHER_FAILURE;
	}
	if(verbose > 1) printf("Got socket\n");

	/* Connect to the host/port. */
	if (connect(sock, (struct sockaddr *)&connectIP, sizeof(connectIP)) < 0) {
		fprintf(stderr, "connect %.100s:%d: %.100s\n", host, port,
			strerror(errno));
#if defined(WIN32)
			closesocket(sock);
#else
			close(sock);
#endif
		free(host);
		return Webfetch::CONNECTION_REFUSED;
	}
	if(verbose > 1) printf("Connected\n");

	/* Successful connection - so far */

	SocksProxyData *socks = context->proxies.GetSocksProxy();
	if (socks->isActive()) {
		err = context->negotiateWithSocks4Proxy(sock, &hostIP);
		if(err != 0) {
			free(host);
			return Webfetch::OTHER_FAILURE;
		}
	}

#if TODO
	// We might want to set non blocking if we want it
	// to be interruptable

#if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
	(void)fcntl(sock, F_SETFL, O_NONBLOCK);
#else /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
	(void)fcntl(sock, F_SETFL, O_NDELAY);
#endif /* O_NONBLOCK && !O_NONBLOCK_BROKEN */

#endif /* TODO */

	free(host);
	serverFD = sock;
	return 0;
}


void Protocol_Handler::shutdown()
{
        if(serverFD >= 0) {
#if defined(WIN32)
                closesocket(serverFD);
#else
                close(serverFD);
#endif

		serverFD = -1;
        }
}

int Protocol_Handler::getNetIOStream()
{
	if(serverFD < 0) return -1;

	hostio = new NetIOStream(serverFD);
	return 0;
}


HTTP_Handler::HTTP_Handler(Webfetch *_context, const char *_host, int _port) :
	Protocol_Handler(_context, _host, _port)
{
	if(port <= 0) port = DEFAULT_HTTP_PORT;
}

HTTP_Handler::~HTTP_Handler()
{
}


void HTTP_Handler::doit(int httpCommand, const char *resource, const UploadData *args)
{
	char	cmd[2000];
	int	null = 0;
	int	n;
	long 	actualSize = 0;
	static char	resp[8193];
	PrintingDataConsumer	printit;

	FILE	*out = NULL;

	hostio->setTimeout(context->getTimeout());

	if(resource == NULL || *resource == '\0') resource = "/";

	if(context->getDataConsumer() == NULL) context->setDataConsumer(&printit);

	context->getDataConsumer()->startStream(resource);

	if(httpCommand == CMD_DELETE)  {
		strcpy(cmd, "DELETE ");
	} else if(httpCommand == CMD_HEAD)  {
		strcpy(cmd, "HEAD ");
	} else if(httpCommand==CMD_PUT || httpCommand==CMD_POST || args!=NULL) {
		if(httpCommand == CMD_PUT) {
			strcpy(cmd, "PUT ");
		} else {
			strcpy(cmd, "POST ");
		}
	} else if(httpCommand == CMD_PROPFIND)  {
		strcpy(cmd, "PROPFIND ");
	} else if(httpCommand == CMD_PROPDEL)  {
		strcpy(cmd, "PROPDEL ");
	} else if(httpCommand == CMD_REPORT)  {
		strcpy(cmd, "REPORT ");
	} else {
		strcpy(cmd, "GET ");
	}

	/* use virtual host for "Host" header.  Default value
	 * is the website, taken either from the call or the
	 * url
	 * NOTE: this is for the deprecated -H option.  The preferred
	 * thing is to put the correct virtual host in the URL, and
	 * specify the alternate host to connect to with -m
	 */
	const char *vhost = "";
	if(context->virtualHost != NULL && *context->virtualHost != '\0') {
		vhost = context->virtualHost;
	} else {
		assert(host != NULL && *host != '\0');
		vhost = host;
	}

	// For HTTP proxy you must prepend host to the URI
	if(context->proxies.GetHTTPProxy()->isActive()) {
		sprintf(&cmd[strlen(cmd)], "http://%s:%d/", vhost, port);
	}
	strcat(cmd, resource);

#if HTTP10
	strcat(cmd, " HTTP/1.0\r\n");
#else
	strcat(cmd, " HTTP/1.1\r\n");
	strcat(cmd, "Host: ");
	strcat(cmd, vhost);
	strcat(cmd, "\r\n");
#endif
	if(context->getAuthorization()) {
		// TODO - other types
		strcat(cmd, "Authorization: Basic ");
		strcat(cmd, context->getAuthorization());
		strcat(cmd, "\r\n");
	}
	if(context->proxies.getAuthorization() != NULL) {
		strcat(cmd, "Proxy-Authorization: Basic ");
		strcat(cmd, context->proxies.getAuthorization());
		strcat(cmd, "\r\n");
	}

	if(context->getCookie()) {
		strcat(cmd, "Cookie: ");
		strcat(cmd, context->getCookie());
		strcat(cmd, "\n");
	}
	CookieJar *cookieJar = context->getCookieJar();
	if(cookieJar != NULL) {
	    for(int ic = 0; ic < cookieJar->ncookies; ic++) {
		Cookie *c = cookieJar->cookies[ic];
		int	ld = strlen(c->domain);
		int	ls = strlen(host);
		if((strcmp(host, c->domain) == 0) ||
		   (strcmp(c->domain, "*") == 0) ||
		   (*c->domain == '.' && ls > ld &&
		    strcmp(&host[ls-ld], c->domain) == 0)) {
			if(verbose > 0) {
				fprintf(stderr, "Sending Cookie: %s=%s, dom=%s\n", c->name, c->value, c->domain);
			}
			strcat(cmd, "Cookie: ");
			strcat(cmd, c->name);
			strcat(cmd, "=");
			strcat(cmd, c->value);
			strcat(cmd, "\n");
		} else {
			if(verbose > 0) {
				fprintf(stderr, "Skiping Cookie: %s=%s, dom=%s\n", c->name, c->value, c->domain);
			}
		}
	    }
	}
	if(context->getFrom()) {
		strcat(cmd, "From: ");
		strcat(cmd, context->getFrom());
		strcat(cmd, "\r\n");
	}
	if(context->getReferer()) {
		strcat(cmd, "Referer: ");
		strcat(cmd, context->getReferer());
		strcat(cmd, "\r\n");
	}

	strcat(cmd, "User-Agent: ");
	strcat(cmd, context->getUserAgent());
	strcat(cmd, "\r\n");
	if(context->getLanguage() && *context->getLanguage() != '\0') {
		strcat(cmd, "Accept-Language: ");
		strcat(cmd, context->getLanguage());
		strcat(cmd, "\r\n");
	}
#if SAMPLE
	strcat(cmd, "Accept: image/jpeg\r\n");
	strcat(cmd, "Connection: Keep-Alive\r\n");
	strcat(cmd, "Pragma: no-cache\r\n");
#endif
	if(args != NULL) {
		// printf("args='%s'\n", args);
		strcat(cmd, "Content-Length: ");
		sprintf(&cmd[strlen(cmd)], "%d\r\n", args->getSize());

		// content type from command line has precedence over args
		const char *ct = context->getContentType();
		if(ct == NULL) ct = args->getContentType();

		if(ct == NULL) {
			// use default
			strcat(cmd, "Content-Type: application/x-www-form-urlencoded\r\n");
		} else if(*ct != '\0') {
			sprintf(&cmd[strlen(cmd)], "Content-Type: %s\r\n", ct);
		}
		/* else - it is an empty string, so we omit it.
		 * Note: This is non-conforming to the RFC, but some lame
		 * servers actually barf if you send it when they do not expect it,
		 * so we need an option to surpress it.
		 */
	}

	// terminate the headers with a blank line
	strcat(cmd, "\r\n");

	if(verbose > 2) fprintf(stdout, "Sending '%s'\n", cmd);
	hostio->send(cmd, strlen(cmd));

	if(args != NULL) {
		if(verbose > 2) fprintf(stdout, "Sending POST data...\n");
		args->send(hostio);
	}


	// Get Header
	int	eol = 1;
	int	bodyIndex = 0;
	int	bodyLen = 0;

	contentLength = 0;
	chunked = 0;
	httpStatus = 0;
	httpMessage = NULL;
	redirect = NULL;

	int okExit = 0;
	while(1) {

		alarm(context->getTimeout());
		n = hostio->readLine(resp, sizeof(resp)-1);
		if(n < 0) break;
		if(n == 0) {
			// printf("got 0 length read\n");
			continue;
		}
		resp[n] = '\0';

		// look for the blank line that ends the header
		if((resp[0] == '\n') ||
		   (resp[0] == '\r' && resp[1] == '\n')) {

			// call the handler in case it wants to mark
			// the end of the headers
			context->handleHeader(&resp[0], n);

			// Was this an HTTP continue header?
			if(httpStatus == HTTP_CONTINUE) {
				// Yup, reset ourselves to get the real
				// response
				contentLength = 0;
				chunked = 0;
				continue;
			}
			okExit = 1;
			break;
		}

		// analyze and deal with it
		context->handleHeader(&resp[0], n);

		if(httpCommand == CMD_HEAD)  {
			// If only doing a HEAD operation, then we
			// treat that as the data to emit.
			context->getDataConsumer()->takeData(resp, n);
		}
	}

	if(okExit == 0) {
		// If we end up here, the server shut us down before the 
		// end of the headers

		// There should be some error recovery 

	}


	/// normalize header information

	/* RFC2068 section 4.4 states that messages must not include 
	 * both Content-Length and be chunked.  If you get both you
	 * must ignore Content-Length
	 */

	if(chunked && contentLength > 0) {
		contentLength = 0;
	}


	// Get Body

	if(httpStatus != HTTP_OK) {
		fprintf(stderr, "HTTP Status: %d", httpStatus);
		if(httpMessage != NULL) fprintf(stderr, " %s", httpMessage);
		fprintf(stderr, "\n");
	}
	if(redirect != NULL) {
		fprintf(stderr, "Redirect to %s\n", redirect);
	}

	if(verbose > 1) {
		fprintf(stderr, "Reading Body (have %d left in buffer)\n",
					hostio->left());
	}

	if(httpCommand == CMD_HEAD) {
		context->getDataConsumer()->endStream();
		return;	// nothing to do
	}

	// Read the data

	long	chunkSize = -1;
	while(1) {
		long nread = sizeof(resp);

		if(contentLength > 0 && actualSize >= contentLength) break;

		if(chunked && chunkSize <= 0) {
			while(1) {
				// read, skipping blank lines
				n = hostio->readLine(resp, sizeof(resp));
				// fprintf(stderr,"wait for chunk def:%s",resp);
				if(n < 0) break;
				if(resp[0] != '\r' || resp[1] != '\n') break;
			}

			if(n < 0) {
				//printf("got %d from readLine, chunkSize\n",n);
				break;
			}

			chunkSize = strtol(&resp[0], NULL, 16);
			if(verbose > 2) {
				printf("Next chunk is %d bytes (%s)\n",
					chunkSize, resp);
			}
			if(chunkSize <= 0) break;

		}
		if(chunked) {
			if(chunkSize < nread) nread = chunkSize;
		}

		// fprintf(stderr, "Calling read %d", nread);
		n = hostio->read(resp, nread);
		// fprintf(stderr, " -> %d\n", n);

		if(n < 0) {
			// fprintf(stderr, "Got eof at %d bytes\n", actualSize);
			break;
		}
		if(n > 0) {
			// resp[n] = '\0';

			context->getDataConsumer()->takeData(resp, n);
			null = 0;
			if(chunkSize > 0) chunkSize -= n;
			actualSize += n;
			if(verbose > 1) {
				if(contentLength > 0) {
					fprintf(stderr, "%d bytes of %d read\n",
						actualSize, contentLength);
				} else {
					fprintf(stderr, "%d bytes read\n",
						actualSize);
				}
			}
		} else {
			// count empty reads
			null++;
			// fprintf(stderr, "Got Null read\n");
			if(null > 2) break;
		}
	}
	context->getDataConsumer()->endStream();
	shutdown();

	if(redirect != NULL && context->getFollowRedirect()) {
		/* NOTE: important to strip args on redirect because
		 * After a POST or DELETE should do a GET, rather than another
		 * of the same operation.
		 */
		context->processURL(CMD_GET, redirect, NULL);
	}
	return;
}

/* Sub classes which override handleHeader must call processHeader
 * to handle simple things
 * The sequence should look like:
 *
 *	MyClass::handleHeader(const char *x, int n) {
 *		if(headerHandler != NULL) {
 *			headerHandler(this, x, n);
 *		}
 *		this->Webfetch::processHeader(x, n);
 *
 *		// my special header processing
 *	}
 */

void HTTP_Handler::processHeader(const char *resp, int /* n */)
{
	// Default processing
	if(strncasecmp(resp, "HTTP/", 5) == 0) {
		// e.g. HTTP/1.1 402 Requires Authentication
		const char *c = &resp[5];
		while(*c && !isspace(*c)) c++;
		while(*c && isspace(*c)) c++;
		httpStatus = 0;
		while(*c && isdigit(*c)) {
			httpStatus = httpStatus * 10 + (*c++ - '0');
		}
		while(*c && isspace(*c)) c++;
		if(*c) {
			const char *msg = c;
			// find end of first line
			while(*c && *c!='\r' && *c!='\n') c++;
			delete httpMessage;
			httpMessage = new char[c-msg+1];
			strncpy(httpMessage, msg, c-msg);
			httpMessage[c-msg] = '\0';
		}
	}

	if(strncasecmp(resp, "Content-Length: ", 16) == 0) {
		contentLength = atol(&resp[16]);
		if(verbose > 1) {
			fprintf(stderr, "Got content length: %d\n",
						contentLength);
		}
	}
	if(strncasecmp(resp, "Transfer-Encoding: chunked", 26) == 0) {
		if(verbose > 1) fprintf(stderr, "Chunked transfer\n");
		chunked = 1;
	}
	if(strncasecmp(resp, "set-cookie: ", 12) == 0) {
		/* Set-Cookie: visit=19991011022825;
		 * expires=Mon-10-2000 02:28:S GMT; path=
		 */
		CookieJar *cj = context->getCookieJar();
		if(cj != NULL) cj->handleCookie(&resp[12], host);
	}
	if(strncasecmp(resp, "location: ", 10) == 0) {
		// copy new resource name and trim trailing CRLF
		redirect = strdup(&resp[10]);
		int l = strlen(redirect) - 1;
		while(l >= 0 && isspace(redirect[l])) {
			redirect[l] = '\0';
			l--;
		}
	}
}


#if HAVE_SSL

HTTPS_Handler::HTTPS_Handler(Webfetch *_context, const char *_host, int _port) :
	HTTP_Handler(_context, _host, _port)
{
	if(_port <= 0) port = DEFAULT_HTTPS_PORT;
}

HTTPS_Handler::~HTTPS_Handler()
{
}

int HTTPS_Handler::getNetIOStream()
{
	// printf("HTTPS_Handler::getNetIOStream()\n");

	SimpleSSL *ssl = SimpleSSL::getSimpleSSL();
	hostio = new NetIOStreamSSL(ssl, serverFD);
	int err = ssl->connect(serverFD);
	if(err != 0) {
		// fprintf(stderr, "Unable to establish SSL connection.\n");
		// SimpleSSL::printErrors();
		err = Webfetch::SSL_CERTIFICATE_FAILURE;
	}
	return err;
}


#endif /* HAVE_SSL */
